from typing import Any, Dict, List, Literal, Optional, Union, AsyncIterable, Iterable
from pydantic import parse_obj_as
import cherwell_pydantic_api._generated.api.models.Trebuchet.WebApi.DataContracts.Forms
import cherwell_pydantic_api.types
from cherwell_pydantic_api.generated_api_utils import GeneratedInterfaceBase


class FormsInterface(GeneratedInterfaceBase):
    async def GetMobileFormForBusObByIdAndPublicIdV1(
        self,
        busobid: cherwell_pydantic_api.types.BusObIDParamType,
        publicid: str,
        foredit: Optional[bool] = None,
        formid: Optional[str] = None,
    ) -> cherwell_pydantic_api._generated.api.models.Trebuchet.WebApi.DataContracts.Forms.MobileFormResponse:
        """Get mobile form by BusObId and Public ID

        Operation that gets a mobile form for a specific Business Object by Business Object ID and Public ID.
         :param busobid: Specify the Business Object ID.
         :param publicid: Specify the Business Object Public ID.
         :param foredit: Flag to get the edit mode version of a form.
         :param formid: Specify the form ID if the default is not desired.
         :return: cherwell_pydantic_api._generated.api.models.Trebuchet.WebApi.DataContracts.Forms.MobileFormResponse
        """
        self.validate_path_param(busobid)
        self.validate_path_param(publicid)
        params = {}
        if foredit is not None:
            params["foredit"] = foredit
        if formid is not None:
            params["formid"] = formid
        response = await self.get(
            f"/api/V1/getmobileformforbusob/busobid/{busobid}/publicid/{publicid}",
            params=params,
        )
        return parse_obj_as(
            cherwell_pydantic_api._generated.api.models.Trebuchet.WebApi.DataContracts.Forms.MobileFormResponse,
            response,
        )

    async def GetMobileFormForBusObByIdAndRecIdV1(
        self,
        busobid: cherwell_pydantic_api.types.BusObIDParamType,
        busobrecid: cherwell_pydantic_api.types.BusObRecID,
        foredit: Optional[bool] = None,
        formid: Optional[str] = None,
    ) -> cherwell_pydantic_api._generated.api.models.Trebuchet.WebApi.DataContracts.Forms.MobileFormResponse:
        """Get mobile form by Business Object ID and Business Object Record ID.

        Operation that gets a mobile form for a specific Business Object by Business Object ID and record ID.
         :param busobid: Specify the Business Object ID.
         :param busobrecid: Specify the Business Object Record ID.
         :param foredit: Flag to get the edit mode version of a form.
         :param formid: Specify the form ID if the default is not desired.
         :return: cherwell_pydantic_api._generated.api.models.Trebuchet.WebApi.DataContracts.Forms.MobileFormResponse
        """
        self.validate_path_param(busobid)
        self.validate_path_param(busobrecid)
        params = {}
        if foredit is not None:
            params["foredit"] = foredit
        if formid is not None:
            params["formid"] = formid
        response = await self.get(
            f"/api/V1/getmobileformforbusob/busobid/{busobid}/busobrecid/{busobrecid}",
            params=params,
        )
        return parse_obj_as(
            cherwell_pydantic_api._generated.api.models.Trebuchet.WebApi.DataContracts.Forms.MobileFormResponse,
            response,
        )

    async def GetMobileFormForBusObByNameAndPublicIdV1(
        self,
        busobname: str,
        publicid: str,
        foredit: Optional[bool] = None,
        formid: Optional[str] = None,
    ) -> cherwell_pydantic_api._generated.api.models.Trebuchet.WebApi.DataContracts.Forms.MobileFormResponse:
        """Get mobile form by Business Object name and Public ID

        Operation that gets a mobile form for a specific Business Object by Business Object name and public ID.
         :param busobname: Specify the Business Object name.
         :param publicid: Specify the Business Object public ID.
         :param foredit: Flag to get the edit mode version of a form.
         :param formid: Specify the form ID if the default is not desired.
         :return: cherwell_pydantic_api._generated.api.models.Trebuchet.WebApi.DataContracts.Forms.MobileFormResponse
        """
        self.validate_path_param(busobname)
        self.validate_path_param(publicid)
        params = {}
        if foredit is not None:
            params["foredit"] = foredit
        if formid is not None:
            params["formid"] = formid
        response = await self.get(
            f"/api/V1/getmobileformforbusob/busobname/{busobname}/publicid/{publicid}",
            params=params,
        )
        return parse_obj_as(
            cherwell_pydantic_api._generated.api.models.Trebuchet.WebApi.DataContracts.Forms.MobileFormResponse,
            response,
        )

    async def GetMobileFormForBusObByNameAndRecIdV1(
        self,
        busobname: str,
        busobrecid: cherwell_pydantic_api.types.BusObRecID,
        foredit: Optional[bool] = None,
        formid: Optional[str] = None,
    ) -> cherwell_pydantic_api._generated.api.models.Trebuchet.WebApi.DataContracts.Forms.MobileFormResponse:
        """Get mobile form by Business Object name and record ID.

        Operation that gets a mobile form for a specific Business Object by Business Object name and record ID.
         :param busobname: Specify the Business Object name.
         :param busobrecid: Specify the Business Object record ID.
         :param foredit: Flag to get the edit mode version of a form.
         :param formid: Specify the form ID if the default is not desired.
         :return: cherwell_pydantic_api._generated.api.models.Trebuchet.WebApi.DataContracts.Forms.MobileFormResponse
        """
        self.validate_path_param(busobname)
        self.validate_path_param(busobrecid)
        params = {}
        if foredit is not None:
            params["foredit"] = foredit
        if formid is not None:
            params["formid"] = formid
        response = await self.get(
            f"/api/V1/getmobileformforbusob/busobname/{busobname}/busobrecid/{busobrecid}",
            params=params,
        )
        return parse_obj_as(
            cherwell_pydantic_api._generated.api.models.Trebuchet.WebApi.DataContracts.Forms.MobileFormResponse,
            response,
        )



from typing import TYPE_CHECKING, Any, AsyncIterable, Iterable, Mapping, NewType, Optional, Union

import httpx
import pydantic
import re


URLType = Union[str, httpx.URL]
CherwellObjectID = NewType("CherwellObjectID", str)
HttpStatusCode = httpx.codes

_re_path_param = re.compile(r'^[a-zA-Z0-9_-]+$')


class ApiBaseModel(pydantic.BaseModel):
    """Base class of all generated API models."""

    # Methods to improve IPython experience

    def __dir__(self):
        return [arg[0] for arg in self.__repr_args__()]

    def _repr_pretty_(self, p, cycle):
        if cycle:
            p.text(f"{self.__class__.__name__}(...)")
        else:
            p.begin_group(2, f"{self.__class__.__name__}(")
            first = True
            for name, value in self.__repr_args__():
                if not first:
                    p.text(',')
                    p.breakable(' ')
                first = False
                if name is not None:
                    p.text(f"{name}=")
                p.pretty(value)
            p.end_group(2, ')')


# This interface is only for type checking.
if TYPE_CHECKING:
    class GeneratedInterfaceBaseType:
        async def get(self, url: URLType, *, params: Optional[Mapping[str, Any]] = None) -> Any:
            ...

        async def post_body(self, url: URLType, *, content: Union[str, bytes, Iterable[bytes], AsyncIterable[bytes]], content_type: str = 'application/json', params: Optional[Mapping[str, Any]] = None) -> Any:
            ...

        async def post_form(self, url: URLType, *, data: Optional[Mapping[str, Any]] = None, params: Optional[Mapping[str, Any]] = None) -> Any:
            ...

        async def post(self, url: URLType) -> Any:
            ...

        async def put(self, url: URLType, *, params: Optional[Mapping[str, Any]] = None) -> Any:
            ...

        async def delete(self, url: URLType, *, params: Optional[Mapping[str, Any]] = None) -> Any:
            ...
else:
    class GeneratedInterfaceBaseType:
        pass


class GeneratedInterfaceBase(GeneratedInterfaceBaseType):
    """Generated Cherwell API interface"""

    def validate_path_param(self, value: str):
        if not _re_path_param.match(value):
            raise ValueError(f"Invalid path parameter: {value!r}")



from typing import Any, Dict, List, Literal, Optional, Union, AsyncIterable, Iterable
from pydantic import parse_obj_as
import cherwell_pydantic_api._generated.api.models.Trebuchet.WebApi.DataContracts.Users
from cherwell_pydantic_api.generated_api_utils import GeneratedInterfaceBase


class UsersInterface(GeneratedInterfaceBase):
    async def DeleteUserBatchV1(
        self,
        request: cherwell_pydantic_api._generated.api.models.Trebuchet.WebApi.DataContracts.Users.UserBatchDeleteRequest,
    ) -> cherwell_pydantic_api._generated.api.models.Trebuchet.WebApi.DataContracts.Users.UserBatchDeleteResponse:
        """Delete a batch of users

        Operation to delete a batch of users. To get record IDs, use "Get a user by login ID" or "Get a user by public id."
         :param request: Request object listing record IDs for users to be deleted and an error flag.
         :return: cherwell_pydantic_api._generated.api.models.Trebuchet.WebApi.DataContracts.Users.UserBatchDeleteResponse
        """
        response = await self.post_body(
            "/api/V1/deleteuserbatch",
            content=request.json(exclude_unset=True, by_alias=True),
        )
        return parse_obj_as(
            cherwell_pydantic_api._generated.api.models.Trebuchet.WebApi.DataContracts.Users.UserBatchDeleteResponse,
            response,
        )

    async def DeleteUserBatchV2(
        self,
        request: cherwell_pydantic_api._generated.api.models.Trebuchet.WebApi.DataContracts.Users.UserBatchDeleteRequest,
    ) -> cherwell_pydantic_api._generated.api.models.Trebuchet.WebApi.DataContracts.Users.UserBatchDeleteV2Response:
        """Delete a batch of users

        Operation to delete a batch of users. To get record IDs, use "Get a user by login ID" or "Get a user by public id."
         :param request: Request object listing record IDs for users to be deleted and an error flag.
         :return: cherwell_pydantic_api._generated.api.models.Trebuchet.WebApi.DataContracts.Users.UserBatchDeleteV2Response
        """
        response = await self.post_body(
            "/api/V2/deleteuserbatch",
            content=request.json(exclude_unset=True, by_alias=True),
        )
        return parse_obj_as(
            cherwell_pydantic_api._generated.api.models.Trebuchet.WebApi.DataContracts.Users.UserBatchDeleteV2Response,
            response,
        )

    async def DeleteUserV1(
        self,
        userrecordid: str,
    ) -> cherwell_pydantic_api._generated.api.models.Trebuchet.WebApi.DataContracts.Users.UserDeleteResponse:
        """Delete a user by record ID

        Operation to delete a user by record ID. To get record IDs, use "Get a user by login ID" or "Get a user by public id."
         :param userrecordid: Specify the record ID of the user you want to delete
         :return: cherwell_pydantic_api._generated.api.models.Trebuchet.WebApi.DataContracts.Users.UserDeleteResponse
        """
        self.validate_path_param(userrecordid)
        response = await self.delete(f"/api/V1/deleteuser/userrecordid/{userrecordid}")
        return parse_obj_as(
            cherwell_pydantic_api._generated.api.models.Trebuchet.WebApi.DataContracts.Users.UserDeleteResponse,
            response,
        )

    async def DeleteUserV2(
        self,
        userrecordid: str,
    ) -> cherwell_pydantic_api._generated.api.models.Trebuchet.WebApi.DataContracts.Users.UserDeleteV2Response:
        """Delete a user by record ID

        Operation to delete a user by record ID. To get record IDs, use "Get a user by login ID" or "Get a user by public id."
         :param userrecordid: Specify the record ID of the user you want to delete
         :return: cherwell_pydantic_api._generated.api.models.Trebuchet.WebApi.DataContracts.Users.UserDeleteV2Response
        """
        self.validate_path_param(userrecordid)
        response = await self.delete(f"/api/V2/deleteuser/userrecordid/{userrecordid}")
        return parse_obj_as(
            cherwell_pydantic_api._generated.api.models.Trebuchet.WebApi.DataContracts.Users.UserDeleteV2Response,
            response,
        )

    async def GetListOfUsers(
        self,
        loginidfilter: Literal["Internal", "Windows", "Both"],
        stoponerror: Optional[bool] = None,
    ) -> cherwell_pydantic_api._generated.api.models.Trebuchet.WebApi.DataContracts.Users.UserListResponse:
        """Get a list of all system users.

        Operation to get a list of all system users.
         :param loginidfilter: Specify the login ID filter to apply to the users list.
         :param stoponerror: Specify whether the operation is interrupted if retrieving any user causes an error.
         :return: cherwell_pydantic_api._generated.api.models.Trebuchet.WebApi.DataContracts.Users.UserListResponse
        """
        params: dict[str, Any] = {"loginidfilter": loginidfilter}
        if stoponerror is not None:
            params["stoponerror"] = stoponerror
        response = await self.get("/api/V1/getlistofusers", params=params)
        return parse_obj_as(
            cherwell_pydantic_api._generated.api.models.Trebuchet.WebApi.DataContracts.Users.UserListResponse,
            response,
        )

    async def GetUserBatchV1(
        self,
        request: cherwell_pydantic_api._generated.api.models.Trebuchet.WebApi.DataContracts.Users.UserBatchReadRequest,
    ) -> cherwell_pydantic_api._generated.api.models.Trebuchet.WebApi.DataContracts.Users.UserBatchReadResponse:
        """Get user information in a batch

        Operation to get user information in a batch. To get record IDs, use "Get a user by login ID" or "Get a user by public id."
         :param request: Request object that lists user record IDs or public IDs of users and an error flag.
         :return: cherwell_pydantic_api._generated.api.models.Trebuchet.WebApi.DataContracts.Users.UserBatchReadResponse
        """
        response = await self.post_body(
            "/api/V1/getuserbatch",
            content=request.json(exclude_unset=True, by_alias=True),
        )
        return parse_obj_as(
            cherwell_pydantic_api._generated.api.models.Trebuchet.WebApi.DataContracts.Users.UserBatchReadResponse,
            response,
        )

    async def GetUserByLoginIdV1(
        self,
        loginid: str,
    ) -> cherwell_pydantic_api._generated.api.models.Trebuchet.WebApi.DataContracts.Users.User:
        """Get a user by login ID

        Operation to get detailed user information by login ID. Use to get user record IDs and account settings, for example. This operation has been deprecated by a V2 operation of the same name, but with query string parameters.
         :param loginid: Specify the user's login ID.
         :return: cherwell_pydantic_api._generated.api.models.Trebuchet.WebApi.DataContracts.Users.User
        """
        self.validate_path_param(loginid)
        response = await self.get(f"/api/V1/getuserbyloginid/loginid/{loginid}")
        return parse_obj_as(
            cherwell_pydantic_api._generated.api.models.Trebuchet.WebApi.DataContracts.Users.User,
            response,
        )

    async def GetUserByLoginIdV2(
        self,
        loginid: str,
        loginidtype: Literal["Internal", "Windows"],
    ) -> cherwell_pydantic_api._generated.api.models.Trebuchet.WebApi.DataContracts.Users.User:
        """Get a user by login ID and login ID type

        Operation to get detailed user information by login ID. Use to get user record IDs and account settings, for example.
         :param loginid: Specify the user's login ID.
         :param loginidtype: Specify the login ID type.
         :return: cherwell_pydantic_api._generated.api.models.Trebuchet.WebApi.DataContracts.Users.User
        """
        params: dict[str, Any] = {"loginid": loginid, "loginidtype": loginidtype}
        response = await self.get("/api/V2/getuserbyloginid", params=params)
        return parse_obj_as(
            cherwell_pydantic_api._generated.api.models.Trebuchet.WebApi.DataContracts.Users.User,
            response,
        )

    async def GetUserByLoginIdV3(
        self,
        loginid: str,
        loginidtype: Literal["Internal", "Windows"],
    ) -> cherwell_pydantic_api._generated.api.models.Trebuchet.WebApi.DataContracts.Users.UserV2:
        """Get a user by login ID and login ID type

        Operation to get detailed user information by login ID. Use to get user record IDs and account settings, for example.
         :param loginid: Specify the user's login ID.
         :param loginidtype: Specify the login ID type.
         :return: cherwell_pydantic_api._generated.api.models.Trebuchet.WebApi.DataContracts.Users.UserV2
        """
        params: dict[str, Any] = {"loginid": loginid, "loginidtype": loginidtype}
        response = await self.get("/api/V3/getuserbyloginid", params=params)
        return parse_obj_as(
            cherwell_pydantic_api._generated.api.models.Trebuchet.WebApi.DataContracts.Users.UserV2,
            response,
        )

    async def GetUserByPublicIdV1(
        self,
        publicid: str,
    ) -> cherwell_pydantic_api._generated.api.models.Trebuchet.WebApi.DataContracts.Users.UserReadResponse:
        """Get a user by public ID

        Operation to get detailed user information by public ID. Use to get user record IDs and account settings, for example.
         :param publicid: Specify the user's public ID.
         :return: cherwell_pydantic_api._generated.api.models.Trebuchet.WebApi.DataContracts.Users.UserReadResponse
        """
        self.validate_path_param(publicid)
        response = await self.get(f"/api/V1/getuserbypublicid/publicid/{publicid}")
        return parse_obj_as(
            cherwell_pydantic_api._generated.api.models.Trebuchet.WebApi.DataContracts.Users.UserReadResponse,
            response,
        )

    async def GetUserByPublicIdV2(
        self,
        publicid: str,
    ) -> cherwell_pydantic_api._generated.api.models.Trebuchet.WebApi.DataContracts.Users.UserReadV2Response:
        """Get a user by public ID

        Operation to get detailed user information by public ID. Use to get user record IDs and account settings, for example.
         :param publicid: Specify the user's public ID.
         :return: cherwell_pydantic_api._generated.api.models.Trebuchet.WebApi.DataContracts.Users.UserReadV2Response
        """
        self.validate_path_param(publicid)
        response = await self.get(f"/api/V2/getuserbypublicid/publicid/{publicid}")
        return parse_obj_as(
            cherwell_pydantic_api._generated.api.models.Trebuchet.WebApi.DataContracts.Users.UserReadV2Response,
            response,
        )

    async def GetUserByRecId(
        self,
        recid: str,
    ) -> cherwell_pydantic_api._generated.api.models.Trebuchet.WebApi.DataContracts.Users.UserV2:
        """Get a user by record ID

        Operation to get detailed user information by record ID.  Use to get user public IDs and account settings, for example.
         :param recid: Specify the user's record ID
         :return: cherwell_pydantic_api._generated.api.models.Trebuchet.WebApi.DataContracts.Users.UserV2
        """
        self.validate_path_param(recid)
        response = await self.get(f"/api/V1/getuserbyrecid/recid/{recid}")
        return parse_obj_as(
            cherwell_pydantic_api._generated.api.models.Trebuchet.WebApi.DataContracts.Users.UserV2,
            response,
        )

    async def SaveUserBatchV1(
        self,
        request: cherwell_pydantic_api._generated.api.models.Trebuchet.WebApi.DataContracts.Users.UserBatchSaveRequest,
    ) -> cherwell_pydantic_api._generated.api.models.Trebuchet.WebApi.DataContracts.Users.UserBatchSaveResponse:
        """Create or update users in a batch

        Operation to create or update users in a batch. To update, specify record ID. To create, leave record ID empty.
         :param request: Request object listing user record IDs and an error flag.
         :return: cherwell_pydantic_api._generated.api.models.Trebuchet.WebApi.DataContracts.Users.UserBatchSaveResponse
        """
        response = await self.post_body(
            "/api/V1/saveuserbatch",
            content=request.json(exclude_unset=True, by_alias=True),
        )
        return parse_obj_as(
            cherwell_pydantic_api._generated.api.models.Trebuchet.WebApi.DataContracts.Users.UserBatchSaveResponse,
            response,
        )

    async def SaveUserBatchV2(
        self,
        request: cherwell_pydantic_api._generated.api.models.Trebuchet.WebApi.DataContracts.Users.UserBatchSaveV2Request,
    ) -> cherwell_pydantic_api._generated.api.models.Trebuchet.WebApi.DataContracts.Users.UserBatchSaveV2Response:
        """Create or update users in a batch

        Operation to create or update users in a batch. To update, specify record ID. To create, leave record ID empty.
         :param request: Request object listing user record IDs and an error flag.
         :return: cherwell_pydantic_api._generated.api.models.Trebuchet.WebApi.DataContracts.Users.UserBatchSaveV2Response
        """
        response = await self.post_body(
            "/api/V2/saveuserbatch",
            content=request.json(exclude_unset=True, by_alias=True),
        )
        return parse_obj_as(
            cherwell_pydantic_api._generated.api.models.Trebuchet.WebApi.DataContracts.Users.UserBatchSaveV2Response,
            response,
        )

    async def SaveUserV1(
        self,
        request: cherwell_pydantic_api._generated.api.models.Trebuchet.WebApi.DataContracts.Users.UserSaveRequest,
    ) -> cherwell_pydantic_api._generated.api.models.Trebuchet.WebApi.DataContracts.Users.UserSaveResponse:
        """Create or update a user

        Operation to create or update a user.  The response is a collection because if you use a public ID, more than one user could be updated since public IDs may not be unique.
         :param request: Request object to specify user parameters and fields with values to be created or updated. The loginId and either the Business Object record ID or Public ID are required.
         :return: cherwell_pydantic_api._generated.api.models.Trebuchet.WebApi.DataContracts.Users.UserSaveResponse
        """
        response = await self.post_body(
            "/api/V1/saveuser", content=request.json(exclude_unset=True, by_alias=True)
        )
        return parse_obj_as(
            cherwell_pydantic_api._generated.api.models.Trebuchet.WebApi.DataContracts.Users.UserSaveResponse,
            response,
        )

    async def SaveUserV2(
        self,
        request: cherwell_pydantic_api._generated.api.models.Trebuchet.WebApi.DataContracts.Users.UserSaveV2Request,
    ) -> cherwell_pydantic_api._generated.api.models.Trebuchet.WebApi.DataContracts.Users.UserSaveV2Response:
        """Create or update a user

        Operation to create or update a user.  The response is a collection because if you use a public ID, more than one user could be updated since public IDs may not be unique.
         :param request: Request object to specify user parameters and fields with values to be created or updated. The loginId and either the Business Object record ID or Public ID are required.
         :return: cherwell_pydantic_api._generated.api.models.Trebuchet.WebApi.DataContracts.Users.UserSaveV2Response
        """
        response = await self.post_body(
            "/api/V2/saveuser", content=request.json(exclude_unset=True, by_alias=True)
        )
        return parse_obj_as(
            cherwell_pydantic_api._generated.api.models.Trebuchet.WebApi.DataContracts.Users.UserSaveV2Response,
            response,
        )



from typing import Iterable, Mapping, Union

from cherwell_pydantic_api._generated.api.models.Trebuchet.WebApi.DataContracts.BusinessObject import SchemaResponse
from cherwell_pydantic_api.bo.valid_schema import ValidSchema
from cherwell_pydantic_api.bo.wrapper import BusinessObjectWrapper
from cherwell_pydantic_api.bo.interfaces import ApiRequesterInterface
from cherwell_pydantic_api.types import BusObID



class BusinessObjectRegistry(Mapping[str, BusinessObjectWrapper]):
    def __init__(self, instance: ApiRequesterInterface):
        self._instance = instance
        self._schemas: dict[str, ValidSchema] = {}
        self._name_to_id: dict[str, str] = {}
        self._wrappers: dict[str, BusinessObjectWrapper] = {}


    def register(self, schema: SchemaResponse):
        if schema.busObId is None:
            raise ValueError('SchemaResponse.busObId is None')
        if schema.name is None:
            raise ValueError('SchemaResponse.name is None')
        busobid = schema.busObId.lower()
        name = schema.name.lower()
        if busobid in self._schemas:
            if self._schemas[busobid] != schema:
                raise ValueError(
                    f'SchemaResponse.busObId {schema.busObId} already registered and new schema is different')
            if name in self._name_to_id:
                if self._name_to_id[name] != busobid:
                    raise ValueError(
                        f'SchemaResponse.name {schema.name} already registered and new busObId is different')
            return
        valid_schema = ValidSchema.from_schema_response(schema)
        self._schemas[busobid] = valid_schema
        self._name_to_id[name] = busobid
        self._wrappers[busobid] = BusinessObjectWrapper(
            valid_schema, self._instance)


    def __getitem__(self, item: Union[BusObID, str]) -> BusinessObjectWrapper:
        """Look up a business object wrapper by name or ID."""
        item_lower = item.lower()
        if item_lower in self._schemas:
            return self._wrappers[item_lower]
        if item_lower in self._name_to_id:
            return self._wrappers[self._name_to_id[item_lower]]
        raise KeyError(item)


    def __iter__(self) -> Iterable[str]:
        """Iterate over the names of the registered business objects. (Not the business object IDs)"""
        return iter(self._name_to_id.keys())


    def __len__(self) -> int:
        """Return the number of registered business objects."""
        return len(self._schemas)

    
    def __getattr__(self, attr: Union[BusObID, str]) -> BusinessObjectWrapper:
        try:
            return self.__getitem__(attr)
        except KeyError:
            raise AttributeError(attr)


    def __repr__(self) -> str:
        return f'{self.__class__.__name__}(instance={self._instance.settings.name})'


    def __dir__(self) -> Iterable[str]:
        return [k for k in self._name_to_id.keys()] + [k for k in super().__dir__()]



# generated by datamodel-codegen:
#   filename:  swagger.json
#   timestamp: 2023-03-19T19:24:29+00:00

from __future__ import annotations

from enum import Enum
from typing import List, Optional

import cherwell_pydantic_api.types as ct
from cherwell_pydantic_api.generated_api_utils import ApiBaseModel, HttpStatusCode

from . import Core


class OneStepActionResponse(ApiBaseModel):
    completed: Optional[bool] = None
    currentPrimaryBusObId: Optional[ct.BusObID] = None
    currentPrimaryBusObRecId: Optional[ct.BusObRecID] = None
    hasNewAccessToken: Optional[bool] = None
    newAccessToken: Optional[str] = None
    errorCode: Optional[str] = None
    errorMessage: Optional[str] = None
    hasError: Optional[bool] = None
    httpStatusCode: Optional[HttpStatusCode] = None


import cherwell_pydantic_api.types as ct


class OneStepActionRequest(ApiBaseModel):
    acquireLicense: Optional[bool] = None
    busObId: Optional[ct.BusObID] = None
    busObRecId: Optional[ct.BusObRecID] = None
    oneStepActionStandInKey: Optional[str] = None
    promptValues: Optional[List[Core.SimplePromptValue]] = None



from typing import Any, Dict, List, Literal, Optional, Union, AsyncIterable, Iterable
from pydantic import parse_obj_as
import cherwell_pydantic_api._generated.api.models.Trebuchet.WebApi.DataContracts.Core
from cherwell_pydantic_api.generated_api_utils import GeneratedInterfaceBase


class CoreInterface(GeneratedInterfaceBase):
    async def DeleteGalleryImageByStandInKeyV1(
        self,
        standinkey: str,
    ) -> None:
        """Delete a gallery image

        Endpoint to delete a gallery image.
         :param standinkey: The StandIn key for the gallery image to delete.
         :return: None"""
        self.validate_path_param(standinkey)
        await self.delete(f"/api/V1/deletegalleryimage/standinkey/{standinkey}")
        return None

    async def GetGalleryImagesFolderV1(
        self,
        scope: str,
        scopeowner: str,
        folder: str,
        links: Optional[bool] = None,
    ) -> cherwell_pydantic_api._generated.api.models.Trebuchet.WebApi.DataContracts.Core.ManagerData:
        """Get gallery images by scope, scopeowner, and folder

        Get gallery images for the specified scope, scopeowner, and folder.
         :param scope: The scope to get gallery images for.
         :param scopeowner: the scopeowner to get gallery images for.
         :param folder: The folder to get gallery images for.
         :param links: Flag to include hyperlinks in results. Default is false.
         :return: cherwell_pydantic_api._generated.api.models.Trebuchet.WebApi.DataContracts.Core.ManagerData
        """
        self.validate_path_param(scope)
        self.validate_path_param(scopeowner)
        self.validate_path_param(folder)
        params = {}
        if links is not None:
            params["links"] = links
        response = await self.get(
            f"/api/V1/getgalleryimages/scope/{scope}/scopeowner/{scopeowner}/folder/{folder}",
            params=params,
        )
        return parse_obj_as(
            cherwell_pydantic_api._generated.api.models.Trebuchet.WebApi.DataContracts.Core.ManagerData,
            response,
        )

    async def GetGalleryImagesScopeOwnerV1(
        self,
        scope: str,
        scopeowner: str,
        links: Optional[bool] = None,
    ) -> cherwell_pydantic_api._generated.api.models.Trebuchet.WebApi.DataContracts.Core.ManagerData:
        """Get gallery images by scope and scopeowner

        Get all gallery images for the specified scope and scope owner.
         :param scope: The scope to get gallery images for.
         :param scopeowner: The scopeowner to get gallery images for.
         :param links: Flag to include hyperlinks in results. Default is false.
         :return: cherwell_pydantic_api._generated.api.models.Trebuchet.WebApi.DataContracts.Core.ManagerData
        """
        self.validate_path_param(scope)
        self.validate_path_param(scopeowner)
        params = {}
        if links is not None:
            params["links"] = links
        response = await self.get(
            f"/api/V1/getgalleryimages/scope/{scope}/scopeowner/{scopeowner}",
            params=params,
        )
        return parse_obj_as(
            cherwell_pydantic_api._generated.api.models.Trebuchet.WebApi.DataContracts.Core.ManagerData,
            response,
        )

    async def GetGalleryImagesScopeV1(
        self,
        scope: str,
        links: Optional[bool] = None,
    ) -> cherwell_pydantic_api._generated.api.models.Trebuchet.WebApi.DataContracts.Core.ManagerData:
        """Get gallery images by scope

        Get all gallery images for the specified scope.
         :param scope: The scope to get the images for.
         :param links: Flag to include hyperlinks in results. Default is false.
         :return: cherwell_pydantic_api._generated.api.models.Trebuchet.WebApi.DataContracts.Core.ManagerData
        """
        self.validate_path_param(scope)
        params = {}
        if links is not None:
            params["links"] = links
        response = await self.get(
            f"/api/V1/getgalleryimages/scope/{scope}", params=params
        )
        return parse_obj_as(
            cherwell_pydantic_api._generated.api.models.Trebuchet.WebApi.DataContracts.Core.ManagerData,
            response,
        )

    async def GetGalleryImagesV1(
        self,
        links: Optional[bool] = None,
    ) -> cherwell_pydantic_api._generated.api.models.Trebuchet.WebApi.DataContracts.Core.ManagerData:
        """Get all gallery images

        Get all the gallery images in the system.
         :param links: Flag to include hyperlinks in results. Default is false.
         :return: cherwell_pydantic_api._generated.api.models.Trebuchet.WebApi.DataContracts.Core.ManagerData
        """
        params = {}
        if links is not None:
            params["links"] = links
        response = await self.get("/api/V1/getgalleryimages", params=params)
        return parse_obj_as(
            cherwell_pydantic_api._generated.api.models.Trebuchet.WebApi.DataContracts.Core.ManagerData,
            response,
        )

    async def GetGalleryImageV1(
        self,
        name: str,
        width: Optional[int] = None,
        height: Optional[int] = None,
    ) -> str:
        """Get built-in images

        Operation that gets built-in images. If you are requesting an icon (.ico), you can specify width and height.
         :param name: Image name and folder location in the Image Manager. Parameter must begin with "[PlugIn]Images;" and then a period-separated list of folders. Example: "[PlugIn]Images;Images.Common.Cherwell.ico".
         :param width: Specify the width (icons only).
         :param height: Specify the height (icons only).
         :return: str"""
        self.validate_path_param(name)
        params = {}
        if width is not None:
            params["width"] = width
        if height is not None:
            params["height"] = height
        return await self.get(f"/api/V1/getgalleryimage/name/{name}", params=params)

    async def GetStoredValuesFolderV1(
        self,
        scope: str,
        scopeowner: str,
        folder: str,
        links: Optional[bool] = None,
    ) -> cherwell_pydantic_api._generated.api.models.Trebuchet.WebApi.DataContracts.Core.ManagerData:
        """Get stored values by folder

        Get stored values for the specified folder.
         :param scope: The scope for which to get stored values.
         :param scopeowner: The scope owner for which to get stored values.
         :param folder: The folder for which to get stored values.
         :param links: Flag to include hyperlinks in results. Default is false.
         :return: cherwell_pydantic_api._generated.api.models.Trebuchet.WebApi.DataContracts.Core.ManagerData
        """
        self.validate_path_param(scope)
        self.validate_path_param(scopeowner)
        self.validate_path_param(folder)
        params = {}
        if links is not None:
            params["links"] = links
        response = await self.get(
            f"/api/V1/storedvalues/scope/{scope}/scopeowner/{scopeowner}/folder/{folder}",
            params=params,
        )
        return parse_obj_as(
            cherwell_pydantic_api._generated.api.models.Trebuchet.WebApi.DataContracts.Core.ManagerData,
            response,
        )

    async def GetStoredValuesScopeOwnerV1(
        self,
        scope: str,
        scopeowner: str,
        links: Optional[bool] = None,
    ) -> cherwell_pydantic_api._generated.api.models.Trebuchet.WebApi.DataContracts.Core.ManagerData:
        """Get stored values by scope owner

        Get stored values for the specified scope and scope owner.
         :param scope: The scope for which to get stored values.
         :param scopeowner: The scope owner for which to get stored values.
         :param links: Flag to include hyperlinks in results. Default is false.
         :return: cherwell_pydantic_api._generated.api.models.Trebuchet.WebApi.DataContracts.Core.ManagerData
        """
        self.validate_path_param(scope)
        self.validate_path_param(scopeowner)
        params = {}
        if links is not None:
            params["links"] = links
        response = await self.get(
            f"/api/V1/storedvalues/scope/{scope}/scopeowner/{scopeowner}", params=params
        )
        return parse_obj_as(
            cherwell_pydantic_api._generated.api.models.Trebuchet.WebApi.DataContracts.Core.ManagerData,
            response,
        )

    async def GetStoredValuesScopeV1(
        self,
        scope: str,
        links: Optional[bool] = None,
    ) -> cherwell_pydantic_api._generated.api.models.Trebuchet.WebApi.DataContracts.Core.ManagerData:
        """Get stored values by scope

        Get all the stored values for the specified scope.
         :param scope: The scope for which to get stored values.
         :param links: Flag to include hyperlinks in results. Default is false.
         :return: cherwell_pydantic_api._generated.api.models.Trebuchet.WebApi.DataContracts.Core.ManagerData
        """
        self.validate_path_param(scope)
        params = {}
        if links is not None:
            params["links"] = links
        response = await self.get(f"/api/V1/storedvalues/scope/{scope}", params=params)
        return parse_obj_as(
            cherwell_pydantic_api._generated.api.models.Trebuchet.WebApi.DataContracts.Core.ManagerData,
            response,
        )

    async def GetStoredValuesV1(
        self,
        links: Optional[bool] = None,
    ) -> cherwell_pydantic_api._generated.api.models.Trebuchet.WebApi.DataContracts.Core.ManagerData:
        """Gets all the stored values in the system

        Get all the stored values in the system.
         :param links: Flag to include hyperlinks in results. Default is false.
         :return: cherwell_pydantic_api._generated.api.models.Trebuchet.WebApi.DataContracts.Core.ManagerData
        """
        params = {}
        if links is not None:
            params["links"] = links
        response = await self.get("/api/V1/storedvalues", params=params)
        return parse_obj_as(
            cherwell_pydantic_api._generated.api.models.Trebuchet.WebApi.DataContracts.Core.ManagerData,
            response,
        )

    async def GetStoredValueV1(
        self,
        standInKey: str,
    ) -> cherwell_pydantic_api._generated.api.models.Trebuchet.WebApi.DataContracts.Core.StoredValueResponse:
        """Get a  stored value

        Get a stored value by its StandIn key.
         :param standInKey: The StandIn key for the Stored Value you would like to retrieve.
         :return: cherwell_pydantic_api._generated.api.models.Trebuchet.WebApi.DataContracts.Core.StoredValueResponse
        """
        self.validate_path_param(standInKey)
        response = await self.get(f"/api/V1/getstoredvalue/standinkey/{standInKey}")
        return parse_obj_as(
            cherwell_pydantic_api._generated.api.models.Trebuchet.WebApi.DataContracts.Core.StoredValueResponse,
            response,
        )

    async def GetViewsV1(
        self,
    ) -> cherwell_pydantic_api._generated.api.models.Trebuchet.WebApi.DataContracts.Core.ViewsResponse:
        """Get a list of the views

        Operation to get a list of views that are configured in the system.
         :return: cherwell_pydantic_api._generated.api.models.Trebuchet.WebApi.DataContracts.Core.ViewsResponse
        """
        response = await self.get("/api/V1/getviews")
        return parse_obj_as(
            cherwell_pydantic_api._generated.api.models.Trebuchet.WebApi.DataContracts.Core.ViewsResponse,
            response,
        )

    async def SaveGalleryImageV1(
        self,
        request: cherwell_pydantic_api._generated.api.models.Trebuchet.WebApi.DataContracts.Core.SaveGalleryImageRequest,
    ) -> cherwell_pydantic_api._generated.api.models.Trebuchet.WebApi.DataContracts.Core.SaveGalleryImageResponse:
        """Create or update a gallery image

        Endpoint to Create or update a gallery image. To create a new gallery image leave the StandIn key blank. To update a gallery image provide the StandIn key of the gallery image you want to update.
        There are three different ImageTypes allowed: Imported, Url, and File. To use the Imported image type, provide the filename in the Name property, with extension, and provide the image data in a Base64 encoded format in the Base64EncodedImageData property. The max file size is 512k.
        To use the Url image type,  provide the full network share path to the file in the Name property, ie: "&#92;&#92;&#92;&#92;&#92;&#92;&#92;&#92;networkshare&#92;&#92;&#92;somefolder&#92;&#92;&#92;somefile.jpg". If the file is not accessible to all users it will not visible to all users.
        To use the File image type, provide the full path to the file in the Name property, ie: "C:&#92;&#92;&#92;somefolder&#92;&#92;&#92;somfile.jpg". If the file is not accessible to all users it will not visible to all users.
        When creating or updating an image, Name and ImageType are always required, and if the image type is "Imported", then the Base64EncodedImageData is also required.
        scope, scopeowner, and folder can all be updated independently.
         :param request: To create a new gallery image leave the StandIn key blank. To update a gallery image provide the StandIn key of the gallery image you want to update.
        There are three different ImageTypes allowed: Imported, Url, and File. To use the Imported image type, provide the filename in the Name property, with extension, and provide the image data in a Base64 encoded format in the Base64EncodedImageData property. The max file size is 512k.
        To use the Url image type,  provide the full network share path to the file in the Name property, ie: "&#92;&#92;&#92;&#92;&#92;&#92;&#92;&#92;networkshare&#92;&#92;&#92;somefolder&#92;&#92;&#92;somefile.jpg". If the file is not accessible to all users it will not visible to all users.
        To use the File image type, provide the full path to the file in the Name property, ie: "C:&#92;&#92;&#92;somefolder&#92;&#92;&#92;somfile.jpg". If the file is not accessible to all users it will not visible to all users.
        When creating or updating an image, Name and ImageType are always required, and if the image type is "Imported", then the Base64EncodedImageData is also required.
        scope, scopeowner, and folder can all be updated independently.
         :return: cherwell_pydantic_api._generated.api.models.Trebuchet.WebApi.DataContracts.Core.SaveGalleryImageResponse
        """
        response = await self.post_body(
            "/api/V1/savegalleryimage",
            content=request.json(exclude_unset=True, by_alias=True),
        )
        return parse_obj_as(
            cherwell_pydantic_api._generated.api.models.Trebuchet.WebApi.DataContracts.Core.SaveGalleryImageResponse,
            response,
        )

    async def SaveStoredValueV1(
        self,
        request: cherwell_pydantic_api._generated.api.models.Trebuchet.WebApi.DataContracts.Core.SaveStoredValueRequest,
    ) -> cherwell_pydantic_api._generated.api.models.Trebuchet.WebApi.DataContracts.Core.StoredValueResponse:
        """Create or update a stored value

        Operation to create or update a stored value. To update, specify the StandIn key for the stored value to update. To create leave StandIn key blank, and provide a name, a scope, a type, and a value.
         :param request: The stored value to create or update. To update include the StandIn key for the associated stored value. To create, name, scope, type, and value are required.
         :return: cherwell_pydantic_api._generated.api.models.Trebuchet.WebApi.DataContracts.Core.StoredValueResponse
        """
        response = await self.post_body(
            "/api/V1/savestoredvalue",
            content=request.json(exclude_unset=True, by_alias=True),
        )
        return parse_obj_as(
            cherwell_pydantic_api._generated.api.models.Trebuchet.WebApi.DataContracts.Core.StoredValueResponse,
            response,
        )

    async def SetCultureV1(
        self,
        culturecode: str,
    ) -> str:
        """Set the culture for the current user

        Operation to update the current users culture by culture code. This returns a new access token that has the updated information in it.
         :param culturecode: The culture code to set for the current user.
         :return: str"""
        self.validate_path_param(culturecode)
        return await self.put(f"/api/V1/setculture/culturecode/{culturecode}")



from marshmallow import fields
import datetime
import re


def from_cw_datetime(value):
    value = value.replace('/', '.')
    try:
        return datetime.datetime.strptime(value, '%d.%m.%Y %H:%M')
    except:
        pass
    try:
        return datetime.datetime.strptime(value, '%d.%m.%Y %H:%M:%S')
    except:
        pass
    return None

def from_cw_date(value):
    value = value.replace('/', '.')
    try:
        return datetime.datetime.strptime(value, '%d.%m.%Y').date()
    except:
        pass
    return None

def to_cw_datetime(value):
    return value.strftime('%d/%m/%Y %H:%M')

def to_cw_date(value):
    return value.strftime('%d/%m/%Y')



class DateTime(fields.DateTime):
    DESERIALIZATION_FUNCS = fields.DateTime.DESERIALIZATION_FUNCS.copy()
    DESERIALIZATION_FUNCS['cherwell'] = from_cw_datetime
    SERIALIZATION_FUNCS = fields.DateTime.SERIALIZATION_FUNCS.copy()
    SERIALIZATION_FUNCS['cherwell'] = to_cw_datetime
    DEFAULT_FORMAT = 'cherwell'

    def _deserialize(self, value, attr, data, **kwargs):
        if not value:
            return None
        return fields.DateTime._deserialize(self, value, attr, data, **kwargs)


class Date(DateTime):
    DESERIALIZATION_FUNCS = fields.Date.DESERIALIZATION_FUNCS.copy()
    DESERIALIZATION_FUNCS['cherwell'] = from_cw_date
    SERIALIZATION_FUNCS = fields.Date.SERIALIZATION_FUNCS.copy()
    SERIALIZATION_FUNCS['cherwell'] = to_cw_date
    DEFAULT_FORMAT = 'cherwell'
    OBJ_TYPE = 'date'


class Time(fields.Time):
    def _deserialize(self, value, attr, data, **kwargs):
        if not value:
            return None
        return fields.Time._deserialize(self, value, attr, data, **kwargs)


class Int(fields.Int):
    _map = str.maketrans("',", "__")
    def _deserialize(self, value, attr, data, **kwargs):
        if not value:
            return None
        value = value.translate(self._map)
        return fields.Int._deserialize(self, value, attr, data, **kwargs)


class Decimal(fields.Decimal):
    _map = str.maketrans("',", "__")
    def _deserialize(self, value, attr, data, **kwargs):
        if not value:
            return None
        value = value.translate(self._map)
        return fields.Int._deserialize(self, value, attr, data, **kwargs)



from typing import List, Optional

from cherwell_pydantic_api._generated.api.models.Trebuchet.WebApi.DataContracts.BusinessObject import (
    FieldDefinition,
    GridDefinition,
    Relationship,
    SchemaResponse,
)
from cherwell_pydantic_api.generated_api_utils import ApiBaseModel
from cherwell_pydantic_api.types import BusObID



class ValidSchema(ApiBaseModel):
    busObId: BusObID
    fieldDefinitions: List[FieldDefinition]
    firstRecIdField: Optional[str] = None
    gridDefinitions: Optional[List[GridDefinition]] = None
    name: str
    recIdFields: Optional[str] = None
    relationships: Optional[List[Relationship]] = None
    stateFieldId: Optional[str] = None
    states: Optional[str] = None

    @classmethod
    def from_schema_response(cls, schema: SchemaResponse):
        if schema.hasError or not schema.busObId or not schema.name or not schema.fieldDefinitions:
            raise ValueError(f'SchemaResponse not valid: {schema}')
        return cls(busObId=schema.busObId,
                   fieldDefinitions=schema.fieldDefinitions,
                   firstRecIdField=schema.firstRecIdField,
                   gridDefinitions=schema.gridDefinitions,
                   name=schema.name,
                   recIdFields=schema.recIdFields,
                   relationships=schema.relationships,
                   stateFieldId=schema.stateFieldId,
                   states=schema.states,)



from .Approval import ApprovalInterface
from .BusinessObject import BusinessObjectInterface
from .Core import CoreInterface
from .Forms import FormsInterface
from .Lifecycle import LifecycleInterface
from .OneStepActions import OneStepActionsInterface
from .Queues import QueuesInterface
from .Searches import SearchesInterface
from .Security import SecurityInterface
from .Service import ServiceInterface
from .Teams import TeamsInterface
from .Users import UsersInterface


class GeneratedInterfaces(
    ApprovalInterface,
    BusinessObjectInterface,
    CoreInterface,
    FormsInterface,
    LifecycleInterface,
    OneStepActionsInterface,
    QueuesInterface,
    SearchesInterface,
    SecurityInterface,
    ServiceInterface,
    TeamsInterface,
    UsersInterface,
):
    pass




import sys
import importlib.abc
import importlib.machinery
import types
from typing import Any, Callable, List, Optional, Sequence, Tuple, Union
from marshmallow import Schema, pre_load, post_load, post_dump, fields as marshmallow_fields
import pprint



class TypeHandler(object):
    """docstring for TypeHandler"""
    def __init__(self, schema, marshmallow_fields='fields', cherwell_fields='cherwell_fields'):
        super(TypeHandler, self).__init__()
        self.schema = schema
        self.marshmallow_fields = marshmallow_fields
        self.cherwell_fields = cherwell_fields

    def resolve(self, field):
        bo_type = field['type']
        ftype = self.marshmallow_fields + '.Str'
        args = []
        if bo_type == 'Text':
            ftype = self.marshmallow_fields + '.Str'
        elif bo_type == 'Number':
            if not field['decimalDigits']:
                ftype = self.cherwell_fields + '.Int'
            else:
                ftype = self.cherwell_fields + '.Decimal'
                args.append('places={0[decimalDigits]}'.format(field))
                args.append('as_string=True')
        elif bo_type == 'DateTime':
            if field['hasDate'] and field['hasTime']:
                ftype = self.cherwell_fields + '.DateTime'
            elif field['hasDate']:
                ftype = self.cherwell_fields + '.Date'
            elif field['hasTime']:
                ftype = self.cherwell_fields + '.Time'
        elif bo_type == 'Logical':
                ftype = self.marshmallow_fields + '.Bool'
        if field['required']:
            args.append('required=True')
        return ftype + '(' + ','.join(args) + ')'



class BusinessObjectDict(dict):
    __slots__ = ['busObId', 'busObPublicId', 'busObRecId', 'fieldIds', 'html', 'schema', 'dirty']

    def __init__(self, schema):
        self.schema = schema
        self.busObId = self.busObPublicId = self.busObRecId = None
        self.fieldIds = {}
        self.html = {}
        self.dirty = []

    def set_dirty(self, key):
        if key in self.dirty:
            self.dirty.remove(key)
        self.dirty.append(key)
        return key

    def clear_dirty(self):
        self.dirty = []

    def __setitem__(self, key, value):
        if key not in self or self[key] != value:
            self.set_dirty(key)
        dict.__setitem__(self, key, value)
        if key in self.html:
            del self.html[key]

    def set_html(self, key, html):
        if key not in self.html or self.html[key] != html:
            self.set_dirty(key)
        self.html[key] = html
        dict.__setitem__(self, key, html)

    def save_change(self):
        if not self.dirty:
            return None
        return type(self.schema)(only=self.dirty).dump(self)



class BusinessObjectSchema(Schema):

    def make_empty(self):
        r = BusinessObjectDict(self)
        r.busObId = self._busObId
        r.fieldIds = self._field_name_to_long_id
        return r

    @pre_load(pass_many=True)
    def unpack(self, data, many, **kwargs):
        fields_hash = dict()
        if data is not None:
            for field in data['fields']:
                fields_hash[field['name']] = field['value']
        return fields_hash

    @post_load(pass_many=True, pass_original=True)
    def make_bo_dict(self, data, original, **kwargs):
        r = BusinessObjectDict(self)
        for k, v in data.items():
            r[k] = v
        if original is not None:
            for field in original['fields']:
                r.fieldIds[field['name']] = field['fieldId']
                if field['html'] is not None:
                    r.html[field['name']] = field['html']
            r.busObId = original['busObId']
            r.busObPublicId = original['busObPublicId']
            r.busObRecId = original['busObRecId']
        r.clear_dirty()
        return r


    @post_dump(pass_many=True, pass_original=True)
    def dump_bo_dict(self, data, original, **kwargs):
        fields = []
        r = {'busObId': original.busObId, 'persist': True, 'fields': fields}
        if original.busObRecId is not None:
            r['busObRecId'] = original.busObRecId
        # First enumerate dirty fields in order of setting them
        for field in original.dirty:
            value = data[field]
            f = { 'dirty': True, 'fieldId': original.fieldIds[field] }
            if field in original.html:
                f['html'] = original.html[field]
                f['value'] = original.html[field]
            else:
                f['value'] = value
            f['name'] = field
            fields.append(f)
        # Then add non-dirty fields
        for field, value in data.items():
            if field in original.dirty:
                continue
            f = { 'dirty': False, 'fieldId': original.fieldIds[field] }
            if field in original.html:
                f['html'] = original.html[field]
                f['value'] = original.html[field]
            else:
                f['value'] = value
            f['name'] = field
            fields.append(f)
        return r



class BusinessObjectBuilder(object):
    """docstring for BusinessObjectBuilder"""
    def __init__(self, connection, path, type_handler_class=TypeHandler):
        super(BusinessObjectBuilder, self).__init__()
        self.connection = connection
        self.path = path
        self.type_handler_class = type_handler_class


    def build_object(self, busObName, type_handler_class=None):
        if type_handler_class is None:
            type_handler_class = self.type_handler_class
        lines = ['# Auto-generated file. Do not edit!',
                 '###################################',
                 '',
                 'from marshmallow import fields',
                 'from {0} import BusinessObjectSchema'.format(__name__),
                 'from {0} import fields as cherwell_fields'.format(__name__),
                 '']
        schema = self.connection.get_bo_schema(busObName = busObName, relationships=True)
        self.last_schema = schema
        type_handler = type_handler_class(schema)
        field_id_to_name = dict()
        field_name_to_long_id = dict()
        lines.append(
r"""class {0[name]}Schema(BusinessObjectSchema):
    _busObId = '{0[busObId]}'
    _stateFieldId = '{0[stateFieldId]}'""".format(schema))
        for field in schema['fieldDefinitions']:
            mtype = type_handler.resolve(field)
            description = field['description'].replace('\n', '').strip()
            if description:
                lines.append('    #{0}'.format(description))
            lines.append('    {0[name]} = {1}'.format(field, mtype))
            field_name_to_long_id[field['name']] = field['fieldId']
            field_id_parts = dict()
            for field_id_part in field['fieldId'].split(','):
                k, v = field_id_part.split(':', 1)
                field_id_parts[k] = v
            if field_id_parts.get('BO', '') == schema['busObId']:
                field_id_to_name[field_id_parts.get('FI', '')] = field['name']
        if schema['states'] is not None:
            lines.append('    _states = ' + repr(schema['states'].split(',')))

        if field_id_to_name:
            lines.append('    _first_rec_id_field = {0}'.format(field_id_to_name[schema['firstRecIdField']]))
            if 'stateFieldId' in schema and schema['stateFieldId'] in field_id_to_name:
                lines.append('    _state_field = {0}'.format(field_id_to_name[schema['stateFieldId']]))
            lines.append('    _field_id_to_name = ' + pprint.pformat(field_id_to_name, indent=8, width=160))
        lines.append('    _field_name_to_long_id = ' + pprint.pformat(field_name_to_long_id, indent=8, width=160))
        lines.append('')
        for relationship in schema['relationships']:
            pass

        lines.append('r"""')
        lines.append(pprint.pformat(schema, width=240))
        lines.append('r"""')
        f = open('{0}/{1}.py'.format(self.path, busObName), 'w')
        for line in lines:
            f.write(line + '\n')
        f.close()
        return lines



class CherwellSchemaMetaPathFinder(importlib.abc.MetaPathFinder):
    """docstring for CherwellSchemaMetaPathFinder"""
    def __init__(self):
        super(CherwellSchemaMetaPathFinder, self).__init__()
        self.schemas = dict()
        
    @classmethod
    def find_spec(cls, fullname: str, path: Optional[Sequence[str]], target: Optional[types.ModuleType] = ...) -> Optional[object]:
        if fullname.startswith(cls.__module__ + '.'):
            print("fullname: {0}".format(fullname))
            schema = fullname[len(cls.__module__) + 1:]
            if '.' in schema:
                schema, ob = schema.split('.')
            else:
                ob = None
            print("find_spec: {0}.{1}, {2}, {3}".format(schema, ob, path, target))
            bob = meta_path_finder.schemas.get(schema, None)
            if bob is None:
                return None
            if ob is None:
                spec = importlib.machinery.ModuleSpec(fullname, None, is_package=True)
                print("spec:{0}".format(spec))
                return spec
            bob.build_object(ob)
            loader = importlib.machinery.SourceFileLoader(fullname, '{0}/{1}.py'.format(bob.path, ob))
            spec = importlib.machinery.ModuleSpec(fullname, loader)
            return spec
        return None

    def register_schema(self, name, bob):
        self.schemas[name] = bob


def _append_meta_path():
    for m in sys.meta_path:
        if isinstance(m, CherwellSchemaMetaPathFinder):
            return m
    m = CherwellSchemaMetaPathFinder()
    sys.meta_path.append(m)
    return m

meta_path_finder = _append_meta_path()




# generated by datamodel-codegen:
#   filename:  swagger.json
#   timestamp: 2023-03-19T19:24:29+00:00

from __future__ import annotations

from enum import Enum
from typing import Dict, List, Optional

from pydantic import Field

import cherwell_pydantic_api.types as ct
from cherwell_pydantic_api.generated_api_utils import ApiBaseModel

from .... import CherwellObjectID
from . import BusinessObject, Core


class FilterInfo(ApiBaseModel):
    fieldId: Optional[str] = None
    operator: Optional[str] = None
    value: Optional[str] = None


import cherwell_pydantic_api.types as ct


class SortInfo(ApiBaseModel):
    fieldId: Optional[str] = None
    sortDirection: Optional[int] = None


import cherwell_pydantic_api.types as ct


class QuickSearchConfigSavedRequest(ApiBaseModel):
    standIn: Optional[str] = None
    busObIds: Optional[List[str]] = None
    isGeneral: Optional[bool] = None


from cherwell_pydantic_api.generated_api_utils import HttpStatusCode


class ChangedOption(Enum):
    None_ = 'None'
    Use = 'Use'
    Display = 'Display'
    UseAndDisplay = 'UseAndDisplay'


class NonFinalStateOption(Enum):
    None_ = 'None'
    Use = 'Use'
    Display = 'Display'
    UseAndDisplay = 'UseAndDisplay'


class SearchAnyWordsOption(Enum):
    None_ = 'None'
    Use = 'Use'
    Display = 'Display'
    UseAndDisplay = 'UseAndDisplay'


class SearchAttachmentsOption(Enum):
    None_ = 'None'
    Use = 'Use'
    Display = 'Display'
    UseAndDisplay = 'UseAndDisplay'


class SearchRelatedOption(Enum):
    None_ = 'None'
    Use = 'Use'
    Display = 'Display'
    UseAndDisplay = 'UseAndDisplay'


class SearchTargetType(Enum):
    BusOb = 'BusOb'
    DocRepository = 'DocRepository'


class SortByOption(Enum):
    None_ = 'None'
    Use = 'Use'
    Display = 'Display'
    UseAndDisplay = 'UseAndDisplay'


import cherwell_pydantic_api.types as ct


class ChangedLimit(ApiBaseModel):
    displayName: Optional[str] = None
    units: Optional[str] = None
    value: Optional[int] = None


import cherwell_pydantic_api.types as ct


class QuickSearchConfigurationRequest(ApiBaseModel):
    busObIds: Optional[List[str]] = None


import cherwell_pydantic_api.types as ct


class QuickSearchByIdRequest(ApiBaseModel):
    busObIds: Optional[List[str]] = None
    isGeneral: Optional[bool] = None
    searchText: Optional[str] = None
    standIn: Optional[str] = None


import cherwell_pydantic_api.types as ct


class SimpleResultsListItem(ApiBaseModel):
    busObId: Optional[ct.BusObID] = None
    busObRecId: Optional[ct.BusObRecID] = None
    docRepositoryItemId: Optional[str] = None
    galleryImage: Optional[str] = None
    links: Optional[List[Core.CherwellLink]] = None
    publicId: Optional[str] = None
    scope: Optional[str] = None
    scopeOwner: Optional[str] = None
    subTitle: Optional[str] = None
    text: Optional[str] = None
    title: Optional[str] = None


import cherwell_pydantic_api.types as ct


class QuickSearchRequest(ApiBaseModel):
    busObIds: Optional[List[str]] = None
    searchText: Optional[str] = None


import cherwell_pydantic_api.types as ct


class QuickSearchSpecificByIdRequest(ApiBaseModel):
    busObIds: Optional[List[str]] = None
    standIn: Optional[str] = None
    ascending: Optional[bool] = None
    isBusObTarget: Optional[bool] = None
    nonFinalState: Optional[bool] = None
    searchAnyWords: Optional[bool] = None
    searchAttachments: Optional[bool] = None
    searchRelated: Optional[bool] = None
    searchText: Optional[str] = None
    selectedChangedLimit: Optional[ChangedLimit] = None
    selectedSortByFieldId: Optional[str] = None
    sortByRelevance: Optional[bool] = None
    specificSearchTargetId: Optional[str] = None
    useSortBy: Optional[bool] = None


import cherwell_pydantic_api.types as ct


class FieldModel(ApiBaseModel):
    caption: Optional[str] = None
    currencyCulture: Optional[str] = None
    currencySymbol: Optional[str] = None
    decimalDigits: Optional[int] = None
    defaultSortOrderAscending: Optional[bool] = None
    displayName: Optional[str] = None
    fieldName: Optional[str] = None
    fullFieldId: Optional[str] = None
    hasDefaultSortField: Optional[bool] = None
    fieldId: Optional[str] = None
    isBinary: Optional[bool] = None
    isCurrency: Optional[bool] = None
    isDateTime: Optional[bool] = None
    isFilterAllowed: Optional[bool] = None
    isLogical: Optional[bool] = None
    isNumber: Optional[bool] = None
    isShortDate: Optional[bool] = None
    isShortTime: Optional[bool] = None
    isVisible: Optional[bool] = None
    sortable: Optional[bool] = None
    sortOrder: Optional[str] = None
    storageName: Optional[str] = None
    wholeDigits: Optional[int] = None


import cherwell_pydantic_api.types as ct


class SearchResultsRow(ApiBaseModel):
    busObId: Optional[ct.BusObID] = None
    busObRecId: Optional[ct.BusObRecID] = None
    links: Optional[List[Core.CherwellLink]] = None
    publicId: Optional[str] = None
    rowColor: Optional[str] = None
    searchResultsFieldValues: Optional[List[BusinessObject.FieldTemplateItem]] = None


import cherwell_pydantic_api.types as ct


class QuickSearchSpecificRequest(ApiBaseModel):
    ascending: Optional[bool] = None
    isBusObTarget: Optional[bool] = None
    nonFinalState: Optional[bool] = None
    searchAnyWords: Optional[bool] = None
    searchAttachments: Optional[bool] = None
    searchRelated: Optional[bool] = None
    searchText: Optional[str] = None
    selectedChangedLimit: Optional[ChangedLimit] = None
    selectedSortByFieldId: Optional[str] = None
    sortByRelevance: Optional[bool] = None
    specificSearchTargetId: Optional[str] = None
    useSortBy: Optional[bool] = None


import cherwell_pydantic_api.types as ct


class Association(ApiBaseModel):
    busObId: Optional[ct.BusObID] = None
    busObName: Optional[str] = None


import cherwell_pydantic_api.types as ct


class SearchItem(ApiBaseModel):
    association: Optional[str] = None
    links: Optional[List[Core.CherwellLink]] = None
    localizedScopeName: Optional[str] = None
    parentFolderId: Optional[str] = None
    scope: Optional[str] = None
    scopeOwner: Optional[str] = None
    searchId: Optional[str] = None
    searchName: Optional[str] = None


class ExportFormat(Enum):
    CSV = 'CSV'
    Excel = 'Excel'
    Tab = 'Tab'
    Word = 'Word'
    CustomSeparator = 'CustomSeparator'
    Json = 'Json'


import cherwell_pydantic_api.types as ct


class ExportSearchResultsRequest(ApiBaseModel):
    customSeparator: Optional[str] = None
    exportFormat: Optional[ExportFormat] = None
    exportTitle: Optional[str] = None
    association: Optional[str] = None
    associationName: Optional[str] = None
    busObId: Optional[ct.BusObID] = None
    customGridDefId: Optional[str] = None
    dateTimeFormatting: Optional[str] = None
    fieldId: Optional[str] = None
    fields: Optional[List[str]] = None
    filters: Optional[List[FilterInfo]] = None
    includeAllFields: Optional[bool] = None
    includeSchema: Optional[bool] = None
    pageNumber: Optional[int] = None
    pageSize: Optional[int] = None
    scope: Optional[str] = None
    scopeOwner: Optional[str] = None
    searchId: Optional[str] = None
    searchName: Optional[str] = None
    searchText: Optional[str] = None
    sorting: Optional[List[SortInfo]] = None
    promptValues: Optional[List[Core.PromptValue]] = None


import cherwell_pydantic_api.types as ct


class StoredSearchRequest(ApiBaseModel):
    associationId: Optional[str] = None
    associationName: Optional[str] = None
    gridId: Optional[str] = None
    includeSchema: Optional[bool] = None
    scope: Optional[str] = None
    scopeOwnerId: Optional[str] = None
    searchId: Optional[str] = None
    searchName: Optional[str] = None


class Type(Enum):
    Text = 'Text'
    Number = 'Number'
    Currency = 'Currency'
    Integer = 'Integer'
    Datetime = 'Datetime'
    Date = 'Date'
    Time = 'Time'
    Logical = 'Logical'


import cherwell_pydantic_api.types as ct


class ColumnSchema(ApiBaseModel):
    name: Optional[str] = None
    fieldId: Optional[str] = None
    type: Optional[Type] = None


import cherwell_pydantic_api.types as ct


class RelatedBusinessObjectRequest(ApiBaseModel):
    allFields: Optional[bool] = None
    customGridId: Optional[str] = None
    fieldsList: Optional[List[str]] = None
    filters: Optional[List[FilterInfo]] = None
    pageNumber: Optional[int] = None
    pageSize: Optional[int] = None
    parentBusObId: Optional[ct.BusObID] = None
    parentBusObRecId: Optional[ct.BusObRecID] = None
    relationshipId: Optional[str] = None
    sorting: Optional[List[SortInfo]] = None
    useDefaultGrid: Optional[bool] = None


import cherwell_pydantic_api.types as ct


class QuickSearchItem(ApiBaseModel):
    ascending: Optional[bool] = None
    changedLimits: Optional[List[ChangedLimit]] = None
    changedOption: Optional[ChangedOption] = None
    displayName: Optional[str] = None
    galleryImage: Optional[str] = None
    hasAnyOptions: Optional[bool] = None
    nonFinalStateOption: Optional[NonFinalStateOption] = None
    searchAnyWordsOption: Optional[SearchAnyWordsOption] = None
    searchAttachmentsOption: Optional[SearchAttachmentsOption] = None
    searchRelatedOption: Optional[SearchRelatedOption] = None
    searchTargetId: Optional[str] = None
    searchTargetType: Optional[SearchTargetType] = None
    selectedChangedLimit: Optional[ChangedLimit] = None
    selectedSortByFieldId: Optional[str] = None
    sortByFields: Optional[Dict[str, str]] = None
    sortByOption: Optional[SortByOption] = None
    watermarkText: Optional[str] = None


import cherwell_pydantic_api.types as ct


class SimpleResultsListGroup(ApiBaseModel):
    isBusObTarget: Optional[bool] = None
    simpleResultsListItems: Optional[List[SimpleResultsListItem]] = None
    subTitle: Optional[str] = None
    targetId: Optional[str] = None
    title: Optional[str] = None
    errorCode: Optional[str] = None
    errorMessage: Optional[str] = None
    hasError: Optional[bool] = None
    httpStatusCode: Optional[HttpStatusCode] = None


import cherwell_pydantic_api.types as ct


class SearchResultsTableResponse(ApiBaseModel):
    columns: Optional[List[FieldModel]] = None
    rows: Optional[List[SearchResultsRow]] = None
    sorting: Optional[List[SortInfo]] = None
    totalRows: Optional[int] = None
    errorCode: Optional[str] = None
    errorMessage: Optional[str] = None
    hasError: Optional[bool] = None
    httpStatusCode: Optional[HttpStatusCode] = None


import cherwell_pydantic_api.types as ct


class SearchFolder(ApiBaseModel):
    association: Optional[str] = None
    childFolders: Optional[List[SearchFolder]] = None
    childItems: Optional[List[SearchItem]] = None
    folderId: Optional[str] = None
    folderName: Optional[str] = None
    links: Optional[List[Core.CherwellLink]] = None
    localizedScopeName: Optional[str] = None
    parentFolderId: Optional[str] = None
    scope: Optional[str] = None
    scopeOwner: Optional[str] = None


import cherwell_pydantic_api.types as ct


class SearchResultsRequest(ApiBaseModel):
    association: Optional[str] = None
    associationName: Optional[str] = None
    busObId: Optional[ct.BusObID] = None
    customGridDefId: Optional[str] = None
    dateTimeFormatting: Optional[str] = None
    fieldId: Optional[str] = None
    fields: Optional[List[str]] = None
    filters: Optional[List[FilterInfo]] = None
    includeAllFields: Optional[bool] = None
    includeSchema: Optional[bool] = None
    pageNumber: Optional[int] = None
    pageSize: Optional[int] = None
    scope: Optional[str] = None
    scopeOwner: Optional[str] = None
    searchId: Optional[str] = None
    searchName: Optional[str] = None
    searchText: Optional[str] = None
    sorting: Optional[List[SortInfo]] = None
    promptValues: Optional[List[Core.PromptValue]] = None


import cherwell_pydantic_api.types as ct


class StoredSearchResults(ApiBaseModel):
    columns: Optional[List[ColumnSchema]] = None
    rows: Optional[List[List[CherwellObjectID]]] = Field(None, readOnly=True)


import cherwell_pydantic_api.types as ct


class RelatedBusinessObjectResponse(ApiBaseModel):
    errorCode: Optional[str] = None
    errorMessage: Optional[str] = None
    hasError: Optional[bool] = None
    links: Optional[List[Core.CherwellLink]] = None
    pageNumber: Optional[int] = None
    pageSize: Optional[int] = None
    parentBusObId: Optional[ct.BusObID] = None
    parentBusObPublicId: Optional[str] = None
    parentBusObRecId: Optional[ct.BusObRecID] = None
    relatedBusinessObjects: Optional[List[BusinessObject.ReadResponse]] = None
    relationshipId: Optional[str] = None
    totalRecords: Optional[int] = None


import cherwell_pydantic_api.types as ct


class QuickSearchConfigurationResponse(ApiBaseModel):
    allowQuickSearch: Optional[bool] = None
    allowSpecificSearch: Optional[bool] = None
    defaultToQuickSearch: Optional[bool] = None
    displayName: Optional[str] = None
    galleryImage: Optional[str] = None
    history: Optional[List[str]] = None
    includeAvailableInSpecific: Optional[bool] = None
    includeQuickSearchInSpecific: Optional[bool] = None
    quickSearchId: Optional[str] = None
    quickSearchItems: Optional[List[QuickSearchItem]] = None
    quickSearchWatermark: Optional[str] = None
    sortByRelevance: Optional[bool] = None
    resolvedQuickSearchWatermark: Optional[str] = None
    scope: Optional[str] = None
    scopeOwner: Optional[str] = None
    specificSearchItems: Optional[List[QuickSearchItem]] = None
    errorCode: Optional[str] = None
    errorMessage: Optional[str] = None
    hasError: Optional[bool] = None
    httpStatusCode: Optional[HttpStatusCode] = None


import cherwell_pydantic_api.types as ct


class SimpleResultsList(ApiBaseModel):
    groups: Optional[List[SimpleResultsListGroup]] = None
    title: Optional[str] = None
    errorCode: Optional[str] = None
    errorMessage: Optional[str] = None
    hasError: Optional[bool] = None
    httpStatusCode: Optional[HttpStatusCode] = None


import cherwell_pydantic_api.types as ct


class QuickSearchResponse(ApiBaseModel):
    searchResultsTable: Optional[SearchResultsTableResponse] = None
    simpleResultsList: Optional[SimpleResultsList] = None
    errorCode: Optional[str] = None
    errorMessage: Optional[str] = None
    hasError: Optional[bool] = None
    httpStatusCode: Optional[HttpStatusCode] = None


import cherwell_pydantic_api.types as ct


class SearchItemResponse(ApiBaseModel):
    root: Optional[SearchFolder] = None
    supportedAssociations: Optional[List[Association]] = None
    errorCode: Optional[str] = None
    errorMessage: Optional[str] = None
    hasError: Optional[bool] = None
    httpStatusCode: Optional[HttpStatusCode] = None


import cherwell_pydantic_api.types as ct


class SearchResultsResponse(ApiBaseModel):
    businessObjects: Optional[List[BusinessObject.ReadResponse]] = None
    hasPrompts: Optional[bool] = None
    links: Optional[List[Core.CherwellLink]] = None
    prompts: Optional[List[Core.Prompt]] = None
    searchResultsFields: Optional[List[FieldModel]] = None
    simpleResults: Optional[SimpleResultsList] = None
    totalRows: Optional[int] = None
    hasMoreRecords: Optional[bool] = None
    errorCode: Optional[str] = None
    errorMessage: Optional[str] = None
    hasError: Optional[bool] = None
    httpStatusCode: Optional[HttpStatusCode] = None


SearchFolder.update_forward_refs()



from typing import Any, Dict, List, Literal, Optional, Union, AsyncIterable, Iterable
from pydantic import parse_obj_as
import pydantic
import cherwell_pydantic_api._generated.api.models.Trebuchet.WebApi.DataContracts.Core
import cherwell_pydantic_api._generated.api.models.Trebuchet.WebApi.DataContracts.Security
from cherwell_pydantic_api.generated_api_utils import GeneratedInterfaceBase


class ServiceInterface(GeneratedInterfaceBase):
    async def GetServiceInfoV1(
        self,
    ) -> cherwell_pydantic_api._generated.api.models.Trebuchet.WebApi.DataContracts.Core.ServiceInfoResponse:
        """Get information about the REST Api and CSM

        Operation to get information about the REST API and CSM.  The response is latest REST API operation version, CSM version, and CSM system date and time.
         :return: cherwell_pydantic_api._generated.api.models.Trebuchet.WebApi.DataContracts.Core.ServiceInfoResponse
        """
        response = await self.get("/api/V1/serviceinfo")
        return parse_obj_as(
            cherwell_pydantic_api._generated.api.models.Trebuchet.WebApi.DataContracts.Core.ServiceInfoResponse,
            response,
        )

    async def LogoutUserV1(self) -> None:
        """Log out user by token

        Operation that logs out the user referenced in the authentication token.
         :return: None"""
        await self.delete("/api/V1/logout")
        return None

    async def Token(
        self,
        grant_type: str,
        client_id: pydantic.SecretStr,
        client_secret: Optional[pydantic.SecretStr] = None,
        username: Optional[str] = None,
        password: Optional[pydantic.SecretStr] = None,
        refresh_token: Optional[str] = None,
        auth_mode: Optional[
            Literal["Internal", "Windows", "LDAP", "SAML", "Auto"]
        ] = None,
        site_name: Optional[str] = None,
    ) -> cherwell_pydantic_api._generated.api.models.Trebuchet.WebApi.DataContracts.Security.TokenResponse:
        """Get an access token

        Operation to request an access token for one of the following authentication modes. Or, you can request an access token using a refresh token. An API client key is required in both cases, and the authentication mode you use must be the mode used by the CSM Browser Client.

        Internal - Use a CSM username and password. If no other mode is specified, Internal mode is used.

        Windows - Uses the server variable LOGON_USER to attempt to find a CSM user.  You can also use domain&#92;username and password.

        LDAP - Uses the LDAP settings configured for CSM and the server variable LOGON_USER to attempt to find a CSM user. You can also use domain&#92;username and password.

        SAML - Uses the SAML settings configured for CSM to validate credentials and find the CSM user.

        Auto - Uses the enabled authentication types configured for CSM.
         :param grant_type: The type of grant being requested: password or refresh_token.
         :param client_id: The API client key for the client making the token request.
         :param client_secret: The API client secret for the native client making the token request.  This is only required for native clients.
         :param username: Specify the login ID for the CSM user who will be granted the access token.
         :param password: Specify the password assigned to the login ID.
         :param refresh_token: Specify the refresh token used to grant another access token.
         :param auth_mode: Specify the Authentication Mode to use for requesting an access token.
         :param site_name: If for portal specify the Site name to use for requesting an access token.
         :return: cherwell_pydantic_api._generated.api.models.Trebuchet.WebApi.DataContracts.Security.TokenResponse
        """
        post = {"grant_type": grant_type, "client_id": client_id.get_secret_value()}
        params = {}
        if client_secret is not None:
            post["client_secret"] = client_secret.get_secret_value()
        if username is not None:
            post["username"] = username
        if password is not None:
            post["password"] = password.get_secret_value()
        if refresh_token is not None:
            post["refresh_token"] = refresh_token
        if auth_mode is not None:
            params["auth_mode"] = auth_mode
        if site_name is not None:
            params["site_name"] = site_name
        response = await self.post_form("/token", params=params, data=post)
        return parse_obj_as(
            cherwell_pydantic_api._generated.api.models.Trebuchet.WebApi.DataContracts.Security.TokenResponse,
            response,
        )



# generated by datamodel-codegen:
#   filename:  swagger.json
#   timestamp: 2023-03-19T19:24:29+00:00

from __future__ import annotations

from datetime import datetime
from enum import Enum
from typing import List, Optional

import cherwell_pydantic_api.types as ct
from cherwell_pydantic_api.generated_api_utils import ApiBaseModel

from . import BusinessObject


class User(ApiBaseModel):
    accountLocked: Optional[bool] = None
    createDateTime: Optional[datetime] = None
    displayName: Optional[str] = None
    error: Optional[str] = None
    errorCode: Optional[str] = None
    fields: Optional[List[BusinessObject.FieldTemplateItem]] = None
    hasError: Optional[bool] = None
    lastPasswordResetDate: Optional[datetime] = None
    lastResetDateTime: Optional[datetime] = None
    ldapRequired: Optional[bool] = None
    passwordNeverExpires: Optional[bool] = None
    publicId: Optional[str] = None
    recordId: Optional[str] = None
    securityGroupId: Optional[str] = None
    shortDisplayName: Optional[str] = None
    userCannotChangePassword: Optional[bool] = None
    userMustResetPasswordAtNextLogin: Optional[bool] = None


import cherwell_pydantic_api.types as ct
from cherwell_pydantic_api.generated_api_utils import HttpStatusCode


class UserV2(ApiBaseModel):
    accountLocked: Optional[bool] = None
    createDateTime: Optional[datetime] = None
    displayName: Optional[str] = None
    fields: Optional[List[BusinessObject.FieldTemplateItem]] = None
    lastPasswordResetDate: Optional[datetime] = None
    lastResetDateTime: Optional[datetime] = None
    ldapRequired: Optional[bool] = None
    passwordNeverExpires: Optional[bool] = None
    publicId: Optional[str] = None
    recordId: Optional[str] = None
    securityGroupId: Optional[str] = None
    shortDisplayName: Optional[str] = None
    userCannotChangePassword: Optional[bool] = None
    userMustResetPasswordAtNextLogin: Optional[bool] = None
    errorCode: Optional[str] = None
    errorMessage: Optional[str] = None
    hasError: Optional[bool] = None
    httpStatusCode: Optional[HttpStatusCode] = None


import cherwell_pydantic_api.types as ct


class UserBatchDeleteRequest(ApiBaseModel):
    stopOnError: Optional[bool] = None
    userRecordIds: Optional[List[str]] = None


import cherwell_pydantic_api.types as ct


class UserDeleteResponse(ApiBaseModel):
    error: Optional[str] = None
    errorCode: Optional[str] = None
    hasError: Optional[bool] = None
    users: Optional[List[User]] = None


import cherwell_pydantic_api.types as ct


class UserDeleteV2Response(ApiBaseModel):
    userRecordId: Optional[str] = None
    errorCode: Optional[str] = None
    errorMessage: Optional[str] = None
    hasError: Optional[bool] = None
    httpStatusCode: Optional[HttpStatusCode] = None


import cherwell_pydantic_api.types as ct


class UserListResponse(ApiBaseModel):
    users: Optional[List[UserV2]] = None
    errorCode: Optional[str] = None
    errorMessage: Optional[str] = None
    hasError: Optional[bool] = None
    httpStatusCode: Optional[HttpStatusCode] = None


import cherwell_pydantic_api.types as ct


class UserReadRequest(ApiBaseModel):
    loginId: Optional[str] = None
    publicId: Optional[str] = None


import cherwell_pydantic_api.types as ct


class UserReadResponse(ApiBaseModel):
    error: Optional[str] = None
    errorCode: Optional[str] = None
    hasError: Optional[bool] = None
    users: Optional[List[User]] = None


import cherwell_pydantic_api.types as ct


class UserSaveRequest(ApiBaseModel):
    accountLocked: Optional[bool] = None
    busObId: Optional[ct.BusObID] = None
    busObPublicId: Optional[str] = None
    busObRecId: Optional[ct.BusObRecID] = None
    displayName: Optional[str] = None
    error: Optional[str] = None
    errorCode: Optional[str] = None
    hasError: Optional[bool] = None
    ldapRequired: Optional[bool] = None
    loginId: Optional[str] = None
    nextPasswordResetDate: Optional[datetime] = None
    password: Optional[str] = None
    passwordNeverExpires: Optional[bool] = None
    securityGroupId: Optional[str] = None
    userCannotChangePassword: Optional[bool] = None
    userInfoFields: Optional[List[BusinessObject.FieldTemplateItem]] = None
    userMustChangePasswordAtNextLogin: Optional[bool] = None
    windowsUserId: Optional[str] = None


import cherwell_pydantic_api.types as ct


class UserSaveResponse(ApiBaseModel):
    busObPublicId: Optional[str] = None
    busObRecId: Optional[ct.BusObRecID] = None
    error: Optional[str] = None
    errorCode: Optional[str] = None
    hasError: Optional[bool] = None


import cherwell_pydantic_api.types as ct


class UserSaveV2Request(ApiBaseModel):
    accountLocked: Optional[bool] = None
    allCultures: Optional[bool] = None
    busObId: Optional[ct.BusObID] = None
    busObPublicId: Optional[str] = None
    busObRecId: Optional[ct.BusObRecID] = None
    displayName: Optional[str] = None
    ldapRequired: Optional[bool] = None
    loginId: Optional[str] = None
    nextPasswordResetDate: Optional[datetime] = None
    password: Optional[str] = None
    passwordNeverExpires: Optional[bool] = None
    securityGroupId: Optional[str] = None
    specificCulture: Optional[str] = None
    userCannotChangePassword: Optional[bool] = None
    userInfoFields: Optional[List[BusinessObject.FieldTemplateItem]] = None
    userMustChangePasswordAtNextLogin: Optional[bool] = None
    windowsUserId: Optional[str] = None


import cherwell_pydantic_api.types as ct


class UserSaveV2Response(ApiBaseModel):
    busObPublicId: Optional[str] = None
    busObRecId: Optional[ct.BusObRecID] = None
    errorCode: Optional[str] = None
    errorMessage: Optional[str] = None
    hasError: Optional[bool] = None
    httpStatusCode: Optional[HttpStatusCode] = None


import cherwell_pydantic_api.types as ct


class UserReadV2Response(ApiBaseModel):
    users: Optional[List[UserV2]] = None
    errorCode: Optional[str] = None
    errorMessage: Optional[str] = None
    hasError: Optional[bool] = None
    httpStatusCode: Optional[HttpStatusCode] = None


import cherwell_pydantic_api.types as ct


class UserBatchDeleteResponse(ApiBaseModel):
    responses: Optional[List[UserDeleteResponse]] = None


import cherwell_pydantic_api.types as ct


class UserBatchDeleteV2Response(ApiBaseModel):
    responses: Optional[List[UserDeleteV2Response]] = None
    errorCode: Optional[str] = None
    errorMessage: Optional[str] = None
    hasError: Optional[bool] = None
    httpStatusCode: Optional[HttpStatusCode] = None


import cherwell_pydantic_api.types as ct


class UserBatchReadRequest(ApiBaseModel):
    readRequests: Optional[List[UserReadRequest]] = None
    stopOnError: Optional[bool] = None


import cherwell_pydantic_api.types as ct


class UserBatchReadResponse(ApiBaseModel):
    responses: Optional[List[UserReadV2Response]] = None
    errorCode: Optional[str] = None
    errorMessage: Optional[str] = None
    hasError: Optional[bool] = None
    httpStatusCode: Optional[HttpStatusCode] = None


import cherwell_pydantic_api.types as ct


class UserBatchSaveRequest(ApiBaseModel):
    saveRequests: Optional[List[UserSaveRequest]] = None
    stopOnError: Optional[bool] = None


import cherwell_pydantic_api.types as ct


class UserBatchSaveResponse(ApiBaseModel):
    responses: Optional[List[UserSaveResponse]] = None


import cherwell_pydantic_api.types as ct


class UserBatchSaveV2Request(ApiBaseModel):
    saveRequests: Optional[List[UserSaveV2Request]] = None
    stopOnError: Optional[bool] = None


import cherwell_pydantic_api.types as ct


class UserBatchSaveV2Response(ApiBaseModel):
    responses: Optional[List[UserSaveV2Response]] = None
    errorCode: Optional[str] = None
    errorMessage: Optional[str] = None
    hasError: Optional[bool] = None
    httpStatusCode: Optional[HttpStatusCode] = None



from typing import Any, Dict, List, Literal, Optional, Union, AsyncIterable, Iterable
from pydantic import parse_obj_as
import cherwell_pydantic_api._generated.api.models.Trebuchet.WebApi.DataContracts.Core
import cherwell_pydantic_api._generated.api.models.Trebuchet.WebApi.DataContracts.OneStepActions
import cherwell_pydantic_api.types
from cherwell_pydantic_api.generated_api_utils import GeneratedInterfaceBase


class OneStepActionsInterface(GeneratedInterfaceBase):
    async def GetOneStepActionsByAssociation_Scope_ScopeOwner_FolderV1(
        self,
        association: str,
        scope: str,
        scopeowner: str,
        folder: str,
        links: Optional[bool] = None,
    ) -> cherwell_pydantic_api._generated.api.models.Trebuchet.WebApi.DataContracts.Core.ManagerData:
        """Get One-Step Actions by Folder

        Operation to get One-Step Actions by Association, Scope, Scope Owner in a specific folder.
         :param association: Business Object association to get One-Step Actions for
         :param scope: Scope to get One-Step Actions for
         :param scopeowner: Scope owner to get One-Step Actions for
         :param folder: Folder to get One-Step Actions from
         :param links: Flag to include hyperlinks in results. Default is false.
         :return: cherwell_pydantic_api._generated.api.models.Trebuchet.WebApi.DataContracts.Core.ManagerData
        """
        self.validate_path_param(association)
        self.validate_path_param(scope)
        self.validate_path_param(scopeowner)
        self.validate_path_param(folder)
        params = {}
        if links is not None:
            params["links"] = links
        response = await self.get(
            f"/api/V1/getonestepactions/association/{association}/scope/{scope}/scopeowner/{scopeowner}/folder/{folder}",
            params=params,
        )
        return parse_obj_as(
            cherwell_pydantic_api._generated.api.models.Trebuchet.WebApi.DataContracts.Core.ManagerData,
            response,
        )

    async def GetOneStepActionsByAssociation_Scope_ScopeOwnerV1(
        self,
        association: str,
        scope: str,
        scopeowner: str,
        links: Optional[bool] = None,
    ) -> cherwell_pydantic_api._generated.api.models.Trebuchet.WebApi.DataContracts.Core.ManagerData:
        """Get One-Step Actions by Scope Owner

        Operation to get One-Step Actions by Association, Scope, Scope Owner
         :param association: Business Object association to get One-Step Actions for
         :param scope: Scope to get One-Step Actions for
         :param scopeowner: Scope owner to get One-Step Actions for
         :param links: Flag to include hyperlinks in results. Default is false.
         :return: cherwell_pydantic_api._generated.api.models.Trebuchet.WebApi.DataContracts.Core.ManagerData
        """
        self.validate_path_param(association)
        self.validate_path_param(scope)
        self.validate_path_param(scopeowner)
        params = {}
        if links is not None:
            params["links"] = links
        response = await self.get(
            f"/api/V1/getonestepactions/association/{association}/scope/{scope}/scopeowner/{scopeowner}",
            params=params,
        )
        return parse_obj_as(
            cherwell_pydantic_api._generated.api.models.Trebuchet.WebApi.DataContracts.Core.ManagerData,
            response,
        )

    async def GetOneStepActionsByAssociation_ScopeV1(
        self,
        association: str,
        scope: str,
        links: Optional[bool] = None,
    ) -> cherwell_pydantic_api._generated.api.models.Trebuchet.WebApi.DataContracts.Core.ManagerData:
        """Get One-Step Actions by Scope

        Operation to get One-Step Actions by Association, Scope
         :param association: Business Object association to get One-Step Actions for
         :param scope: Scope to get One-Step Actions for
         :param links: Flag to include hyperlinks in results. Default is false.
         :return: cherwell_pydantic_api._generated.api.models.Trebuchet.WebApi.DataContracts.Core.ManagerData
        """
        self.validate_path_param(association)
        self.validate_path_param(scope)
        params = {}
        if links is not None:
            params["links"] = links
        response = await self.get(
            f"/api/V1/getonestepactions/association/{association}/scope/{scope}",
            params=params,
        )
        return parse_obj_as(
            cherwell_pydantic_api._generated.api.models.Trebuchet.WebApi.DataContracts.Core.ManagerData,
            response,
        )

    async def GetOneStepActionsByAssociationV1(
        self,
        association: str,
        links: Optional[bool] = None,
    ) -> cherwell_pydantic_api._generated.api.models.Trebuchet.WebApi.DataContracts.Core.ManagerData:
        """Get One-Step Actions by Association

        Operation to get One-Step Actions by Association
         :param association: Business Object association to get One-Step Actions for
         :param links: Flag to include hyperlinks in results. Default is false.
         :return: cherwell_pydantic_api._generated.api.models.Trebuchet.WebApi.DataContracts.Core.ManagerData
        """
        self.validate_path_param(association)
        params = {}
        if links is not None:
            params["links"] = links
        response = await self.get(
            f"/api/V1/getonestepactions/association/{association}", params=params
        )
        return parse_obj_as(
            cherwell_pydantic_api._generated.api.models.Trebuchet.WebApi.DataContracts.Core.ManagerData,
            response,
        )

    async def GetOneStepActionsV1(
        self,
        links: Optional[bool] = None,
    ) -> cherwell_pydantic_api._generated.api.models.Trebuchet.WebApi.DataContracts.Core.ManagerData:
        """Get One-Step Actions by default Association

        Operation to get One-Step Actions by default Association
         :param links: Flag to include hyperlinks in results. Default is false.
         :return: cherwell_pydantic_api._generated.api.models.Trebuchet.WebApi.DataContracts.Core.ManagerData
        """
        params = {}
        if links is not None:
            params["links"] = links
        response = await self.get("/api/V1/getonestepactions", params=params)
        return parse_obj_as(
            cherwell_pydantic_api._generated.api.models.Trebuchet.WebApi.DataContracts.Core.ManagerData,
            response,
        )

    async def RunOneStepActionByKeyForRecordByRecIdV1(
        self,
        standinkey: str,
        busobid: cherwell_pydantic_api.types.BusObIDParamType,
        busobrecid: cherwell_pydantic_api.types.BusObRecID,
    ) -> cherwell_pydantic_api._generated.api.models.Trebuchet.WebApi.DataContracts.OneStepActions.OneStepActionResponse:
        """Run a One-Step Action for a Business Object record

        Operation to run a One-Step Action for a Business Object record by Business Object ID and Business Object Record ID.
         :param standinkey: The key to find the One-Step Action to run
         :param busobid: Specify the Business Object ID.
         :param busobrecid: Specify the Business Object record ID.
         :return: cherwell_pydantic_api._generated.api.models.Trebuchet.WebApi.DataContracts.OneStepActions.OneStepActionResponse
        """
        self.validate_path_param(standinkey)
        self.validate_path_param(busobid)
        self.validate_path_param(busobrecid)
        response = await self.get(
            f"/api/V1/runonestepaction/standinkey/{standinkey}/busobid/{busobid}/busobrecid/{busobrecid}"
        )
        return parse_obj_as(
            cherwell_pydantic_api._generated.api.models.Trebuchet.WebApi.DataContracts.OneStepActions.OneStepActionResponse,
            response,
        )

    async def RunOneStepActionByStandInKeyV1(
        self,
        standinkey: str,
    ) -> cherwell_pydantic_api._generated.api.models.Trebuchet.WebApi.DataContracts.OneStepActions.OneStepActionResponse:
        """Run a stand alone One-Step Action

        Operation to run a One-Step Action that doesn't run against a Business Object Record.
         :param standinkey: The key to find the One-Step Action to run
         :return: cherwell_pydantic_api._generated.api.models.Trebuchet.WebApi.DataContracts.OneStepActions.OneStepActionResponse
        """
        self.validate_path_param(standinkey)
        response = await self.get(f"/api/V1/runonestepaction/standinkey/{standinkey}")
        return parse_obj_as(
            cherwell_pydantic_api._generated.api.models.Trebuchet.WebApi.DataContracts.OneStepActions.OneStepActionResponse,
            response,
        )

    async def RunOneStepActionV1(
        self,
        request: cherwell_pydantic_api._generated.api.models.Trebuchet.WebApi.DataContracts.OneStepActions.OneStepActionRequest,
    ) -> cherwell_pydantic_api._generated.api.models.Trebuchet.WebApi.DataContracts.OneStepActions.OneStepActionResponse:
        """Run a One-Step Action using a OneStepActionRequest

        Operation to run a One-Step Action using a OneStepActionRequest. This request is used to start a One-Step Action run with additional information such as prompt values.
         :param request: Request object containing all the properties need to start a One-Step Action.
         :return: cherwell_pydantic_api._generated.api.models.Trebuchet.WebApi.DataContracts.OneStepActions.OneStepActionResponse
        """
        response = await self.post_body(
            "/api/V1/runonestepaction",
            content=request.json(exclude_unset=True, by_alias=True),
        )
        return parse_obj_as(
            cherwell_pydantic_api._generated.api.models.Trebuchet.WebApi.DataContracts.OneStepActions.OneStepActionResponse,
            response,
        )



from typing import Any, Dict, List, Literal, Optional, Union, AsyncIterable, Iterable
from pydantic import parse_obj_as
import cherwell_pydantic_api._generated.api.models.Trebuchet.WebApi.DataContracts.Core
import cherwell_pydantic_api._generated.api.models.Trebuchet.WebApi.DataContracts.Queues
from cherwell_pydantic_api.generated_api_utils import GeneratedInterfaceBase


class QueuesInterface(GeneratedInterfaceBase):
    async def AddItemToQueueV1(
        self,
        request: cherwell_pydantic_api._generated.api.models.Trebuchet.WebApi.DataContracts.Queues.AddItemToQueueRequest,
    ) -> cherwell_pydantic_api._generated.api.models.Trebuchet.WebApi.DataContracts.Queues.AddItemToQueueResponse:
        """Add a Business Object to a queue

        Operation to add a Business Object to a queue
         :param request: Request object containing all properties necessary to add an item to a queue. All properties are required. The standin key defines the queue to which we want to add the Business Object.
         :return: cherwell_pydantic_api._generated.api.models.Trebuchet.WebApi.DataContracts.Queues.AddItemToQueueResponse
        """
        response = await self.post_body(
            "/api/V1/additemtoqueue",
            content=request.json(exclude_unset=True, by_alias=True),
        )
        return parse_obj_as(
            cherwell_pydantic_api._generated.api.models.Trebuchet.WebApi.DataContracts.Queues.AddItemToQueueResponse,
            response,
        )

    async def CheckInQueueItemV1(
        self,
        request: cherwell_pydantic_api._generated.api.models.Trebuchet.WebApi.DataContracts.Queues.CheckInQueueItemRequest,
    ) -> cherwell_pydantic_api._generated.api.models.Trebuchet.WebApi.DataContracts.Queues.CheckInQueueItemResponse:
        """Check a Business Object in to a queue

        Operation to check in a queue item
         :param request: The request object for checking in an item to a queue. All properties are required except for historyNotes
         :return: cherwell_pydantic_api._generated.api.models.Trebuchet.WebApi.DataContracts.Queues.CheckInQueueItemResponse
        """
        response = await self.post_body(
            "/api/V1/checkinqueueitem",
            content=request.json(exclude_unset=True, by_alias=True),
        )
        return parse_obj_as(
            cherwell_pydantic_api._generated.api.models.Trebuchet.WebApi.DataContracts.Queues.CheckInQueueItemResponse,
            response,
        )

    async def CheckOutQueueItemV1(
        self,
        request: cherwell_pydantic_api._generated.api.models.Trebuchet.WebApi.DataContracts.Queues.CheckOutQueueItemRequest,
    ) -> cherwell_pydantic_api._generated.api.models.Trebuchet.WebApi.DataContracts.Queues.CheckOutQueueItemResponse:
        """Check a Business Object out of a queue

        Operation to check out a queue item
         :param request: The request object for checking out an item from a queue. All properties are required except for historyNotes
         :return: cherwell_pydantic_api._generated.api.models.Trebuchet.WebApi.DataContracts.Queues.CheckOutQueueItemResponse
        """
        response = await self.post_body(
            "/api/V1/checkoutqueueitem",
            content=request.json(exclude_unset=True, by_alias=True),
        )
        return parse_obj_as(
            cherwell_pydantic_api._generated.api.models.Trebuchet.WebApi.DataContracts.Queues.CheckOutQueueItemResponse,
            response,
        )

    async def GetQueuesFolderV1(
        self,
        scope: str,
        scopeowner: str,
        folder: str,
        links: Optional[bool] = None,
    ) -> cherwell_pydantic_api._generated.api.models.Trebuchet.WebApi.DataContracts.Core.ManagerData:
        """Get available queues.

        Get available queues for a specific Business Object type based on scope, scope owner, and folder.
         :param scope: The scope to get available queues for.
         :param scopeowner: The scope owner to get available queues for.
         :param folder: The folder to get available queues for.  This has to be the folder ID which can be retrieved by doing a getqueues operation without the folder including links option then the links will have the folder IDs.
         :param links: Whether or not to include links.
         :return: cherwell_pydantic_api._generated.api.models.Trebuchet.WebApi.DataContracts.Core.ManagerData
        """
        self.validate_path_param(scope)
        self.validate_path_param(scopeowner)
        self.validate_path_param(folder)
        params = {}
        if links is not None:
            params["links"] = links
        response = await self.get(
            f"/api/V1/getqueues/scope/{scope}/scopeowner/{scopeowner}/folder/{folder}",
            params=params,
        )
        return parse_obj_as(
            cherwell_pydantic_api._generated.api.models.Trebuchet.WebApi.DataContracts.Core.ManagerData,
            response,
        )

    async def GetQueuesScopeOwnerV1(
        self,
        scope: str,
        scopeowner: str,
        links: Optional[bool] = None,
    ) -> cherwell_pydantic_api._generated.api.models.Trebuchet.WebApi.DataContracts.Core.ManagerData:
        """Get available queues.

        Get available queues for a specific Business Object type based on scope, and scope owner.
         :param scope: The scope to get available queues for.
         :param scopeowner: The scope owner to get available queues for.
         :param links: Whether or not to include links.
         :return: cherwell_pydantic_api._generated.api.models.Trebuchet.WebApi.DataContracts.Core.ManagerData
        """
        self.validate_path_param(scope)
        self.validate_path_param(scopeowner)
        params = {}
        if links is not None:
            params["links"] = links
        response = await self.get(
            f"/api/V1/getqueues/scope/{scope}/scopeowner/{scopeowner}", params=params
        )
        return parse_obj_as(
            cherwell_pydantic_api._generated.api.models.Trebuchet.WebApi.DataContracts.Core.ManagerData,
            response,
        )

    async def GetQueuesScopeV1(
        self,
        scope: str,
        links: Optional[bool] = None,
    ) -> cherwell_pydantic_api._generated.api.models.Trebuchet.WebApi.DataContracts.Core.ManagerData:
        """Get available queues.

        Get available queues for a specific Business Object type based on scope.
         :param scope: The scope to get available queues for.
         :param links: Whether or not to include links.
         :return: cherwell_pydantic_api._generated.api.models.Trebuchet.WebApi.DataContracts.Core.ManagerData
        """
        self.validate_path_param(scope)
        params = {}
        if links is not None:
            params["links"] = links
        response = await self.get(f"/api/V1/getqueues/scope/{scope}", params=params)
        return parse_obj_as(
            cherwell_pydantic_api._generated.api.models.Trebuchet.WebApi.DataContracts.Core.ManagerData,
            response,
        )

    async def GetQueuesV1(
        self,
        links: Optional[bool] = None,
    ) -> cherwell_pydantic_api._generated.api.models.Trebuchet.WebApi.DataContracts.Core.ManagerData:
        """Get available queues.

        Get available queues for a specific Business Object.
         :param links: Whether or not to include links.
         :return: cherwell_pydantic_api._generated.api.models.Trebuchet.WebApi.DataContracts.Core.ManagerData
        """
        params = {}
        if links is not None:
            params["links"] = links
        response = await self.get("/api/V1/getqueues", params=params)
        return parse_obj_as(
            cherwell_pydantic_api._generated.api.models.Trebuchet.WebApi.DataContracts.Core.ManagerData,
            response,
        )

    async def RemoveItemFromQueueV1(
        self,
        request: cherwell_pydantic_api._generated.api.models.Trebuchet.WebApi.DataContracts.Queues.RemoveItemFromQueueRequest,
    ) -> cherwell_pydantic_api._generated.api.models.Trebuchet.WebApi.DataContracts.Queues.RemoveItemFromQueueResponse:
        """Remove an item from a queue

        Operation to remove an item from a queue
         :param request: The request object to remove an item from a queue. All properties are required except for historyNotes
         :return: cherwell_pydantic_api._generated.api.models.Trebuchet.WebApi.DataContracts.Queues.RemoveItemFromQueueResponse
        """
        response = await self.post_body(
            "/api/V1/removeitemfromqueue",
            content=request.json(exclude_unset=True, by_alias=True),
        )
        return parse_obj_as(
            cherwell_pydantic_api._generated.api.models.Trebuchet.WebApi.DataContracts.Queues.RemoveItemFromQueueResponse,
            response,
        )






class BusinessObjectBase(object):
    """docstring for BusinessObjectBase"""
    def __init__(self, connection):
        super(BusinessObjectBase, self).__init__()
        self._conn = connection
        



# generated by datamodel-codegen:
#   filename:  swagger.json
#   timestamp: 2023-03-19T19:24:29+00:00

from __future__ import annotations

from enum import Enum
from typing import List, Optional

import cherwell_pydantic_api.types as ct
from cherwell_pydantic_api.generated_api_utils import ApiBaseModel

from . import BusinessObject


class AddUserToTeamRequest(ApiBaseModel):
    teamId: Optional[str] = None
    userIsTeamManager: Optional[bool] = None
    userRecordId: Optional[str] = None


import cherwell_pydantic_api.types as ct
from cherwell_pydantic_api.generated_api_utils import HttpStatusCode


class AddUserToTeamResponse(ApiBaseModel):
    errorCode: Optional[str] = None
    errorMessage: Optional[str] = None
    hasError: Optional[bool] = None
    httpStatusCode: Optional[HttpStatusCode] = None


import cherwell_pydantic_api.types as ct


class Team(ApiBaseModel):
    teamId: Optional[str] = None
    teamName: Optional[str] = None


import cherwell_pydantic_api.types as ct


class TeamsV2Response(ApiBaseModel):
    teams: Optional[List[Team]] = None
    errorCode: Optional[str] = None
    errorMessage: Optional[str] = None
    hasError: Optional[bool] = None
    httpStatusCode: Optional[HttpStatusCode] = None


class TeamType(Enum):
    User = 'User'
    CustomerWorkgroup = 'CustomerWorkgroup'


import cherwell_pydantic_api.types as ct


class TeamResponse(ApiBaseModel):
    description: Optional[str] = None
    emailAlias: Optional[str] = None
    fields: Optional[List[BusinessObject.FieldTemplateItem]] = None
    image: Optional[str] = None
    name: Optional[str] = None
    teamId: Optional[str] = None
    teamType: Optional[TeamType] = None
    errorCode: Optional[str] = None
    errorMessage: Optional[str] = None
    hasError: Optional[bool] = None
    httpStatusCode: Optional[HttpStatusCode] = None


import cherwell_pydantic_api.types as ct


class RemoveUserFromTeamResponse(ApiBaseModel):
    teamId: Optional[str] = None
    userRecordId: Optional[str] = None
    errorCode: Optional[str] = None
    errorMessage: Optional[str] = None
    hasError: Optional[bool] = None
    httpStatusCode: Optional[HttpStatusCode] = None


import cherwell_pydantic_api.types as ct


class RemoveCustomerFromWorkgroupResponse(ApiBaseModel):
    workgroupId: Optional[str] = None
    customerRecordId: Optional[str] = None
    errorCode: Optional[str] = None
    errorMessage: Optional[str] = None
    hasError: Optional[bool] = None
    httpStatusCode: Optional[HttpStatusCode] = None


import cherwell_pydantic_api.types as ct


class TeamSaveRequest(ApiBaseModel):
    description: Optional[str] = None
    emailAlias: Optional[str] = None
    image: Optional[str] = None
    teamId: Optional[str] = None
    teamName: Optional[str] = None
    teamType: Optional[TeamType] = None


import cherwell_pydantic_api.types as ct


class TeamSaveResponse(ApiBaseModel):
    teamId: Optional[str] = None
    errorCode: Optional[str] = None
    errorMessage: Optional[str] = None
    hasError: Optional[bool] = None
    httpStatusCode: Optional[HttpStatusCode] = None


import cherwell_pydantic_api.types as ct


class SaveTeamMemberRequest(ApiBaseModel):
    isTeamManager: Optional[bool] = None
    setAsDefaultTeam: Optional[bool] = None
    teamId: Optional[str] = None
    userRecId: Optional[str] = None


import cherwell_pydantic_api.types as ct


class SaveTeamMemberResponse(ApiBaseModel):
    errorCode: Optional[str] = None
    errorMessage: Optional[str] = None
    hasError: Optional[bool] = None
    httpStatusCode: Optional[HttpStatusCode] = None


import cherwell_pydantic_api.types as ct


class SaveWorkgroupMemberRequest(ApiBaseModel):
    customerRecordId: Optional[str] = None
    workgroupId: Optional[str] = None
    customerIsWorkgroupManager: Optional[bool] = None


import cherwell_pydantic_api.types as ct


class SaveWorkgroupMemberResponse(ApiBaseModel):
    errorCode: Optional[str] = None
    errorMessage: Optional[str] = None
    hasError: Optional[bool] = None
    httpStatusCode: Optional[HttpStatusCode] = None


import cherwell_pydantic_api.types as ct


class AddUserToTeamByBatchRequest(ApiBaseModel):
    addUserToTeamRequests: Optional[List[AddUserToTeamRequest]] = None
    stopOnError: Optional[bool] = None


import cherwell_pydantic_api.types as ct


class AddUserToTeamByBatchResponse(ApiBaseModel):
    responses: Optional[List[AddUserToTeamResponse]] = None
    errorCode: Optional[str] = None
    errorMessage: Optional[str] = None
    hasError: Optional[bool] = None
    httpStatusCode: Optional[HttpStatusCode] = None


import cherwell_pydantic_api.types as ct


class TeamsResponse(ApiBaseModel):
    error: Optional[str] = None
    errorCode: Optional[str] = None
    hasError: Optional[bool] = None
    teams: Optional[List[Team]] = None



from typing import Any, Dict, List, Literal, Optional, Union, AsyncIterable, Iterable
from pydantic import parse_obj_as
import cherwell_pydantic_api._generated.api.models.Trebuchet.WebApi.DataContracts.Users
import cherwell_pydantic_api._generated.api.models.Trebuchet.WebApi.DataContracts.Security
import cherwell_pydantic_api.types
from cherwell_pydantic_api.generated_api_utils import GeneratedInterfaceBase


class SecurityInterface(GeneratedInterfaceBase):
    async def GetClientSecuritySettingsV1(
        self,
        applicationtype: Literal[
            "NotSet",
            "RichClient",
            "BrowserClient",
            "BrowserPortal",
            "MobileClient",
            "ServiceMonitor",
        ],
    ) -> cherwell_pydantic_api._generated.api.models.Trebuchet.WebApi.DataContracts.Security.ClientSecuritySettingsResponse:
        """Get client security settings

        Operation to get the configured client security settings. Returns true if internal, Windows, LDAP, or SAML are enabled as authentication methods.
         :param applicationtype: The type of CSM application to return security settings for.  Application type is Desktop Client, Browser Client, Browser Portal or Cherwell Mobile.
         :return: cherwell_pydantic_api._generated.api.models.Trebuchet.WebApi.DataContracts.Security.ClientSecuritySettingsResponse
        """
        self.validate_path_param(applicationtype)
        response = await self.get(
            f"/api/V1/getclientsecuritysettings/applicationtype/{applicationtype}"
        )
        return parse_obj_as(
            cherwell_pydantic_api._generated.api.models.Trebuchet.WebApi.DataContracts.Security.ClientSecuritySettingsResponse,
            response,
        )

    async def GetRolesV1(
        self,
    ) -> cherwell_pydantic_api._generated.api.models.Trebuchet.WebApi.DataContracts.Security.RoleReadResponse:
        """Get all available Roles

        Operation to get all available Roles.
         :return: cherwell_pydantic_api._generated.api.models.Trebuchet.WebApi.DataContracts.Security.RoleReadResponse
        """
        response = await self.get("/api/V1/getroles")
        return parse_obj_as(
            cherwell_pydantic_api._generated.api.models.Trebuchet.WebApi.DataContracts.Security.RoleReadResponse,
            response,
        )

    async def GetRolesV2(
        self,
    ) -> cherwell_pydantic_api._generated.api.models.Trebuchet.WebApi.DataContracts.Security.RoleReadV2Response:
        """Get all available Roles

        Operation to get all available Roles.
         :return: cherwell_pydantic_api._generated.api.models.Trebuchet.WebApi.DataContracts.Security.RoleReadV2Response
        """
        response = await self.get("/api/V2/getroles")
        return parse_obj_as(
            cherwell_pydantic_api._generated.api.models.Trebuchet.WebApi.DataContracts.Security.RoleReadV2Response,
            response,
        )

    async def GetSecurityGroupBusinessObjectPermissionsByBusObIdV1(
        self,
        groupid: str,
        busObId: cherwell_pydantic_api.types.BusObIDParamType,
    ) -> List[
        cherwell_pydantic_api._generated.api.models.Trebuchet.WebApi.DataContracts.Security.BusinessObjectPermission
    ]:
        """Get Business Object permissions by Security Group

        Operation to get specific Business Object permissions for a Security Group.
         :param groupid: Specify the Security Group ID.
         :param busObId: Specify the Business Object ID.
         :return: List[cherwell_pydantic_api._generated.api.models.Trebuchet.WebApi.DataContracts.Security.BusinessObjectPermission]
        """
        self.validate_path_param(groupid)
        self.validate_path_param(busObId)
        response = await self.get(
            f"/api/V1/getsecuritygroupbusinessobjectpermissions/groupid/{groupid}/busobid/{busObId}"
        )
        return parse_obj_as(
            List[
                cherwell_pydantic_api._generated.api.models.Trebuchet.WebApi.DataContracts.Security.BusinessObjectPermission
            ],
            response,
        )

    async def GetSecurityGroupBusinessObjectPermissionsByBusObIdV2(
        self,
        groupid: str,
        busObId: cherwell_pydantic_api.types.BusObIDParamType,
    ) -> cherwell_pydantic_api._generated.api.models.Trebuchet.WebApi.DataContracts.Security.GetSecurityGroupBusinessObjectPermissionsResponse:
        """Get Business Object permissions by Security Group

        Operation to get specific Business Object permissions for a Security Group.
         :param groupid: Specify the Security Group ID.
         :param busObId: Specify the Business Object ID.
         :return: cherwell_pydantic_api._generated.api.models.Trebuchet.WebApi.DataContracts.Security.GetSecurityGroupBusinessObjectPermissionsResponse
        """
        self.validate_path_param(groupid)
        self.validate_path_param(busObId)
        response = await self.get(
            f"/api/V2/getsecuritygroupbusinessobjectpermissions/groupid/{groupid}/busobid/{busObId}"
        )
        return parse_obj_as(
            cherwell_pydantic_api._generated.api.models.Trebuchet.WebApi.DataContracts.Security.GetSecurityGroupBusinessObjectPermissionsResponse,
            response,
        )

    async def GetSecurityGroupBusinessObjectPermissionsByBusObNameV1(
        self,
        groupname: str,
        busobname: str,
    ) -> List[
        cherwell_pydantic_api._generated.api.models.Trebuchet.WebApi.DataContracts.Security.BusinessObjectPermission
    ]:
        """Get Business Object permissions by Security Group

        Operation to get specific Business Object permissions for a Security Group.
         :param groupname: Specify the Security Group name.
         :param busobname: Specify the Business Object name.
         :return: List[cherwell_pydantic_api._generated.api.models.Trebuchet.WebApi.DataContracts.Security.BusinessObjectPermission]
        """
        self.validate_path_param(groupname)
        self.validate_path_param(busobname)
        response = await self.get(
            f"/api/V1/getsecuritygroupbusinessobjectpermissions/groupname/{groupname}/busobname/{busobname}"
        )
        return parse_obj_as(
            List[
                cherwell_pydantic_api._generated.api.models.Trebuchet.WebApi.DataContracts.Security.BusinessObjectPermission
            ],
            response,
        )

    async def GetSecurityGroupBusinessObjectPermissionsByBusObNameV2(
        self,
        groupname: str,
        busobname: str,
    ) -> cherwell_pydantic_api._generated.api.models.Trebuchet.WebApi.DataContracts.Security.GetSecurityGroupBusinessObjectPermissionsResponse:
        """Get Business Object permissions by Security Group

        Operation to get specific Business Object permissions for a Security Group.
         :param groupname: Specify the Security Group name.
         :param busobname: Specify the Business Object name.
         :return: cherwell_pydantic_api._generated.api.models.Trebuchet.WebApi.DataContracts.Security.GetSecurityGroupBusinessObjectPermissionsResponse
        """
        self.validate_path_param(groupname)
        self.validate_path_param(busobname)
        response = await self.get(
            f"/api/V2/getsecuritygroupbusinessobjectpermissions/groupname/{groupname}/busobname/{busobname}"
        )
        return parse_obj_as(
            cherwell_pydantic_api._generated.api.models.Trebuchet.WebApi.DataContracts.Security.GetSecurityGroupBusinessObjectPermissionsResponse,
            response,
        )

    async def GetSecurityGroupBusinessObjectPermissionsForCurrentUserByBusObIdV1(
        self,
        busObId: cherwell_pydantic_api.types.BusObIDParamType,
    ) -> List[
        cherwell_pydantic_api._generated.api.models.Trebuchet.WebApi.DataContracts.Security.BusinessObjectPermission
    ]:
        """Get Business Object permission for current user

        Operation to get Business Object permissions for the currently logged-in user's Security Group.
         :param busObId: Specify the Business Object ID.
         :return: List[cherwell_pydantic_api._generated.api.models.Trebuchet.WebApi.DataContracts.Security.BusinessObjectPermission]
        """
        self.validate_path_param(busObId)
        response = await self.get(
            f"/api/V1/getsecuritygroupbusinessobjectpermissionsforcurrentuserbybusobid/busobid/{busObId}"
        )
        return parse_obj_as(
            List[
                cherwell_pydantic_api._generated.api.models.Trebuchet.WebApi.DataContracts.Security.BusinessObjectPermission
            ],
            response,
        )

    async def GetSecurityGroupBusinessObjectPermissionsForCurrentUserByBusObIdV2(
        self,
        busObId: cherwell_pydantic_api.types.BusObIDParamType,
    ) -> cherwell_pydantic_api._generated.api.models.Trebuchet.WebApi.DataContracts.Security.GetSecurityGroupBusinessObjectPermissionsResponse:
        """Get Business Object permission for current user

        Operation to get Business Object permissions for the currently logged-in user's Security Group.
         :param busObId: Specify the Business Object ID.
         :return: cherwell_pydantic_api._generated.api.models.Trebuchet.WebApi.DataContracts.Security.GetSecurityGroupBusinessObjectPermissionsResponse
        """
        self.validate_path_param(busObId)
        response = await self.get(
            f"/api/V2/getsecuritygroupbusinessobjectpermissionsforcurrentuserbybusobid/busobid/{busObId}"
        )
        return parse_obj_as(
            cherwell_pydantic_api._generated.api.models.Trebuchet.WebApi.DataContracts.Security.GetSecurityGroupBusinessObjectPermissionsResponse,
            response,
        )

    async def GetSecurityGroupBusinessObjectPermissionsForCurrentUserByBusObNameV1(
        self,
        busobname: str,
    ) -> List[
        cherwell_pydantic_api._generated.api.models.Trebuchet.WebApi.DataContracts.Security.BusinessObjectPermission
    ]:
        """Get Business Object permissions for current user

        Operation to get Business Object permissions for currently logged in user's Security Group.
         :param busobname: Specify the Business Object name.
         :return: List[cherwell_pydantic_api._generated.api.models.Trebuchet.WebApi.DataContracts.Security.BusinessObjectPermission]
        """
        self.validate_path_param(busobname)
        response = await self.get(
            f"/api/V1/getsecuritygroupbusinessobjectpermissionsforcurrentuserbybusobname/busobname/{busobname}"
        )
        return parse_obj_as(
            List[
                cherwell_pydantic_api._generated.api.models.Trebuchet.WebApi.DataContracts.Security.BusinessObjectPermission
            ],
            response,
        )

    async def GetSecurityGroupBusinessObjectPermissionsForCurrentUserByBusObNameV2(
        self,
        busobname: str,
    ) -> cherwell_pydantic_api._generated.api.models.Trebuchet.WebApi.DataContracts.Security.GetSecurityGroupBusinessObjectPermissionsResponse:
        """Get Business Object permissions for current user

        Operation to get Business Object permissions for currently logged in user's Security Group.
         :param busobname: Specify the Business Object name.
         :return: cherwell_pydantic_api._generated.api.models.Trebuchet.WebApi.DataContracts.Security.GetSecurityGroupBusinessObjectPermissionsResponse
        """
        self.validate_path_param(busobname)
        response = await self.get(
            f"/api/V2/getsecuritygroupbusinessobjectpermissionsforcurrentuserbybusobname/busobname/{busobname}"
        )
        return parse_obj_as(
            cherwell_pydantic_api._generated.api.models.Trebuchet.WebApi.DataContracts.Security.GetSecurityGroupBusinessObjectPermissionsResponse,
            response,
        )

    async def GetSecurityGroupCategoriesV1(
        self,
    ) -> List[
        cherwell_pydantic_api._generated.api.models.Trebuchet.WebApi.DataContracts.Security.RightCategory
    ]:
        """Get all Security Group categories

        Operation to get IDs and names for all available Security Group categories.
         :return: List[cherwell_pydantic_api._generated.api.models.Trebuchet.WebApi.DataContracts.Security.RightCategory]
        """
        response = await self.get("/api/V1/getsecuritygroupcategories")
        return parse_obj_as(
            List[
                cherwell_pydantic_api._generated.api.models.Trebuchet.WebApi.DataContracts.Security.RightCategory
            ],
            response,
        )

    async def GetSecurityGroupCategoriesV2(
        self,
    ) -> cherwell_pydantic_api._generated.api.models.Trebuchet.WebApi.DataContracts.Security.SecurityRightCategoriesResponse:
        """Get all Security Group categories

        Operation to get IDs and names for all available Security Group categories.
         :return: cherwell_pydantic_api._generated.api.models.Trebuchet.WebApi.DataContracts.Security.SecurityRightCategoriesResponse
        """
        response = await self.get("/api/V2/getsecuritygroupcategories")
        return parse_obj_as(
            cherwell_pydantic_api._generated.api.models.Trebuchet.WebApi.DataContracts.Security.SecurityRightCategoriesResponse,
            response,
        )

    async def GetSecurityGroupRightsByGroupIdAndCategoryIdV1(
        self,
        groupid: str,
        categoryid: str,
    ) -> List[
        cherwell_pydantic_api._generated.api.models.Trebuchet.WebApi.DataContracts.Security.Right
    ]:
        """Get permissions for a Security Group by category

        Operation to get permissions for a Security Group by category. To get Security Group IDs, use "Get all available Security Groups." To get Security Group category IDs, use "Get all Security Group categories."
         :param groupid: Specify the Security Group ID
         :param categoryid: Specify the Security Group category ID
         :return: List[cherwell_pydantic_api._generated.api.models.Trebuchet.WebApi.DataContracts.Security.Right]
        """
        self.validate_path_param(groupid)
        self.validate_path_param(categoryid)
        response = await self.get(
            f"/api/V1/getsecuritygrouprights/groupid/{groupid}/categoryid/{categoryid}"
        )
        return parse_obj_as(
            List[
                cherwell_pydantic_api._generated.api.models.Trebuchet.WebApi.DataContracts.Security.Right
            ],
            response,
        )

    async def GetSecurityGroupRightsByGroupIdAndCategoryIdV2(
        self,
        groupid: str,
        categoryid: str,
    ) -> cherwell_pydantic_api._generated.api.models.Trebuchet.WebApi.DataContracts.Security.SecurityRightsResponse:
        """Get permissions for a Security Group by category

        Operation to get permissions for a Security Group by category. To get Security Group IDs, use "Get all available Security Groups." To get Security Group category IDs, use "Get all Security Group categories."
         :param groupid: Specify the Security Group ID
         :param categoryid: Specify the Security Group category ID
         :return: cherwell_pydantic_api._generated.api.models.Trebuchet.WebApi.DataContracts.Security.SecurityRightsResponse
        """
        self.validate_path_param(groupid)
        self.validate_path_param(categoryid)
        response = await self.get(
            f"/api/V2/getsecuritygrouprights/groupid/{groupid}/categoryid/{categoryid}"
        )
        return parse_obj_as(
            cherwell_pydantic_api._generated.api.models.Trebuchet.WebApi.DataContracts.Security.SecurityRightsResponse,
            response,
        )

    async def GetSecurityGroupRightsByGroupNameAndCategoryNameV1(
        self,
        groupname: str,
        categoryname: str,
    ) -> List[
        cherwell_pydantic_api._generated.api.models.Trebuchet.WebApi.DataContracts.Security.Right
    ]:
        """Get permissions for a Security Group by category

        Operation to get permissions for a Security Group by category.
         :param groupname: Specify the Security Group name.
         :param categoryname: Specify the Security Group category name.
         :return: List[cherwell_pydantic_api._generated.api.models.Trebuchet.WebApi.DataContracts.Security.Right]
        """
        self.validate_path_param(groupname)
        self.validate_path_param(categoryname)
        response = await self.get(
            f"/api/V1/getsecuritygrouprights/groupname/{groupname}/categoryname/{categoryname}"
        )
        return parse_obj_as(
            List[
                cherwell_pydantic_api._generated.api.models.Trebuchet.WebApi.DataContracts.Security.Right
            ],
            response,
        )

    async def GetSecurityGroupRightsByGroupNameAndCategoryNameV2(
        self,
        groupname: str,
        categoryname: str,
    ) -> cherwell_pydantic_api._generated.api.models.Trebuchet.WebApi.DataContracts.Security.SecurityRightsResponse:
        """Get permissions for a Security Group by category

        Operation to get permissions for a Security Group by category.
         :param groupname: Specify the Security Group name.
         :param categoryname: Specify the Security Group category name.
         :return: cherwell_pydantic_api._generated.api.models.Trebuchet.WebApi.DataContracts.Security.SecurityRightsResponse
        """
        self.validate_path_param(groupname)
        self.validate_path_param(categoryname)
        response = await self.get(
            f"/api/V2/getsecuritygrouprights/groupname/{groupname}/categoryname/{categoryname}"
        )
        return parse_obj_as(
            cherwell_pydantic_api._generated.api.models.Trebuchet.WebApi.DataContracts.Security.SecurityRightsResponse,
            response,
        )

    async def GetSecurityGroupRightsForCurrentUserByCategoryIdV1(
        self,
        categoryid: str,
    ) -> List[
        cherwell_pydantic_api._generated.api.models.Trebuchet.WebApi.DataContracts.Security.Right
    ]:
        """Get current user's permissions by Security Group category by ID

        Operation to get permissions for the current user's Security Group by category. To get Security Group category IDs, use "Get all Security Group categories."
         :param categoryid: Specify the Security Group category ID.
         :return: List[cherwell_pydantic_api._generated.api.models.Trebuchet.WebApi.DataContracts.Security.Right]
        """
        self.validate_path_param(categoryid)
        response = await self.get(
            f"/api/V1/getsecuritygrouprightsforcurrentuserbycategoryid/categoryid/{categoryid}"
        )
        return parse_obj_as(
            List[
                cherwell_pydantic_api._generated.api.models.Trebuchet.WebApi.DataContracts.Security.Right
            ],
            response,
        )

    async def GetSecurityGroupRightsForCurrentUserByCategoryIdV2(
        self,
        categoryid: str,
    ) -> cherwell_pydantic_api._generated.api.models.Trebuchet.WebApi.DataContracts.Security.SecurityRightsResponse:
        """Get current user's permissions by Security Group category by ID

        Operation to get permissions for the current user's Security Group by category. To get Security Group category IDs, use "Get all Security Group categories."
         :param categoryid: Specify the Security Group category ID.
         :return: cherwell_pydantic_api._generated.api.models.Trebuchet.WebApi.DataContracts.Security.SecurityRightsResponse
        """
        self.validate_path_param(categoryid)
        response = await self.get(
            f"/api/V2/getsecuritygrouprightsforcurrentuserbycategoryid/categoryid/{categoryid}"
        )
        return parse_obj_as(
            cherwell_pydantic_api._generated.api.models.Trebuchet.WebApi.DataContracts.Security.SecurityRightsResponse,
            response,
        )

    async def GetSecurityGroupRightsForCurrentUserByCategoryNameV1(
        self,
        categoryname: str,
    ) -> List[
        cherwell_pydantic_api._generated.api.models.Trebuchet.WebApi.DataContracts.Security.Right
    ]:
        """Get current user's permissions by Security Group category by name

        Operation to get permissions for the current user's Security Group by category.
         :param categoryname: Specify the Security Group category name.
         :return: List[cherwell_pydantic_api._generated.api.models.Trebuchet.WebApi.DataContracts.Security.Right]
        """
        self.validate_path_param(categoryname)
        response = await self.get(
            f"/api/V1/getsecuritygrouprightsforcurrentuserbycategoryname/categoryname/{categoryname}"
        )
        return parse_obj_as(
            List[
                cherwell_pydantic_api._generated.api.models.Trebuchet.WebApi.DataContracts.Security.Right
            ],
            response,
        )

    async def GetSecurityGroupRightsForCurrentUserByCategoryNameV2(
        self,
        categoryname: str,
    ) -> cherwell_pydantic_api._generated.api.models.Trebuchet.WebApi.DataContracts.Security.SecurityRightsResponse:
        """Get current user's permissions by Security Group category by name

        Operation to get permissions for the current user's Security Group by category.
         :param categoryname: Specify the Security Group category name.
         :return: cherwell_pydantic_api._generated.api.models.Trebuchet.WebApi.DataContracts.Security.SecurityRightsResponse
        """
        self.validate_path_param(categoryname)
        response = await self.get(
            f"/api/V2/getsecuritygrouprightsforcurrentuserbycategoryname/categoryname/{categoryname}"
        )
        return parse_obj_as(
            cherwell_pydantic_api._generated.api.models.Trebuchet.WebApi.DataContracts.Security.SecurityRightsResponse,
            response,
        )

    async def GetSecurityGroupsV1(
        self,
    ) -> cherwell_pydantic_api._generated.api.models.Trebuchet.WebApi.DataContracts.Security.SecurityGroupResponse:
        """Get all available Security Groups

        Operation to get IDs, names, and descriptions for all available Security Groups.
         :return: cherwell_pydantic_api._generated.api.models.Trebuchet.WebApi.DataContracts.Security.SecurityGroupResponse
        """
        response = await self.get("/api/V1/getsecuritygroups")
        return parse_obj_as(
            cherwell_pydantic_api._generated.api.models.Trebuchet.WebApi.DataContracts.Security.SecurityGroupResponse,
            response,
        )

    async def GetSecurityGroupsV2(
        self,
    ) -> cherwell_pydantic_api._generated.api.models.Trebuchet.WebApi.DataContracts.Security.SecurityGroupV2Response:
        """Get all available Security Groups

        Operation to get IDs, names, and descriptions for all available Security Groups.
         :return: cherwell_pydantic_api._generated.api.models.Trebuchet.WebApi.DataContracts.Security.SecurityGroupV2Response
        """
        response = await self.get("/api/V2/getsecuritygroups")
        return parse_obj_as(
            cherwell_pydantic_api._generated.api.models.Trebuchet.WebApi.DataContracts.Security.SecurityGroupV2Response,
            response,
        )

    async def GetUsersInSecurityGroupV1(
        self,
        groupid: str,
    ) -> List[
        cherwell_pydantic_api._generated.api.models.Trebuchet.WebApi.DataContracts.Users.User
    ]:
        """Get users in a Security Group

        Operation to get the users in a specified Security Group. Use "Get all available Security Groups" to get Security Group record IDs.
         :param groupid: Specify the Security Group ID.
         :return: List[cherwell_pydantic_api._generated.api.models.Trebuchet.WebApi.DataContracts.Users.User]
        """
        self.validate_path_param(groupid)
        response = await self.get(f"/api/V1/getusersinsecuritygroup/groupid/{groupid}")
        return parse_obj_as(
            List[
                cherwell_pydantic_api._generated.api.models.Trebuchet.WebApi.DataContracts.Users.User
            ],
            response,
        )

    async def GetUsersInSecurityGroupV2(
        self,
        groupid: str,
    ) -> cherwell_pydantic_api._generated.api.models.Trebuchet.WebApi.DataContracts.Users.UserReadV2Response:
        """Get users in a Security Group

        Operation to get the users in a specified Security Group. Use "Get all available Security Groups" to get Security Group record IDs.
         :param groupid: Specify the Security Group ID.
         :return: cherwell_pydantic_api._generated.api.models.Trebuchet.WebApi.DataContracts.Users.UserReadV2Response
        """
        self.validate_path_param(groupid)
        response = await self.get(f"/api/V2/getusersinsecuritygroup/groupid/{groupid}")
        return parse_obj_as(
            cherwell_pydantic_api._generated.api.models.Trebuchet.WebApi.DataContracts.Users.UserReadV2Response,
            response,
        )



from typing import Any, Dict, List, Literal, Optional, Union, AsyncIterable, Iterable
from pydantic import parse_obj_as
import cherwell_pydantic_api._generated.api.models.Trebuchet.WebApi.DataContracts.Teams
from cherwell_pydantic_api.generated_api_utils import GeneratedInterfaceBase


class TeamsInterface(GeneratedInterfaceBase):
    async def AddUserToTeamByBatchV1(
        self,
        request: cherwell_pydantic_api._generated.api.models.Trebuchet.WebApi.DataContracts.Teams.AddUserToTeamByBatchRequest,
    ) -> cherwell_pydantic_api._generated.api.models.Trebuchet.WebApi.DataContracts.Teams.AddUserToTeamByBatchResponse:
        """Add users to a team by batch

        Operation to add users to a Team by batch. To get internal IDs for users, use “Get User Information in a Batch.” To get a Team's internal ID, use "Get all available Teams."
         :param request: Request object to specify a list of add user to team request objects.
         :return: cherwell_pydantic_api._generated.api.models.Trebuchet.WebApi.DataContracts.Teams.AddUserToTeamByBatchResponse
        """
        response = await self.post_body(
            "/api/V1/addusertoteambybatch",
            content=request.json(exclude_unset=True, by_alias=True),
        )
        return parse_obj_as(
            cherwell_pydantic_api._generated.api.models.Trebuchet.WebApi.DataContracts.Teams.AddUserToTeamByBatchResponse,
            response,
        )

    async def AddUserToTeamV1(
        self,
        dataRequest: cherwell_pydantic_api._generated.api.models.Trebuchet.WebApi.DataContracts.Teams.AddUserToTeamRequest,
    ) -> None:
        """Add a user to a team

        Operation to add a user to a Team. To get the user's internal ID, use "Get a user by login ID" or "Get a user by public ID." To get a Team's internal ID, use "Get all available Teams."
         :param dataRequest: Request object to specify user and team values.
         :return: None"""
        await self.post_body(
            "/api/V1/addusertoteam",
            content=dataRequest.json(exclude_unset=True, by_alias=True),
        )
        return None

    async def AddUserToTeamV2(
        self,
        dataRequest: cherwell_pydantic_api._generated.api.models.Trebuchet.WebApi.DataContracts.Teams.AddUserToTeamRequest,
    ) -> cherwell_pydantic_api._generated.api.models.Trebuchet.WebApi.DataContracts.Teams.AddUserToTeamResponse:
        """Add a user to a team

        Operation to add a user to a Team. To get the user's internal ID, use "Get a user by login ID" or "Get a user by public ID." To get a Team's internal ID, use "Get all available Teams."
         :param dataRequest: Request object to specify user and team values.
         :return: cherwell_pydantic_api._generated.api.models.Trebuchet.WebApi.DataContracts.Teams.AddUserToTeamResponse
        """
        response = await self.post_body(
            "/api/V2/addusertoteam",
            content=dataRequest.json(exclude_unset=True, by_alias=True),
        )
        return parse_obj_as(
            cherwell_pydantic_api._generated.api.models.Trebuchet.WebApi.DataContracts.Teams.AddUserToTeamResponse,
            response,
        )

    async def DeleteTeamV1(
        self,
        teamid: str,
    ) -> None:
        """Delete a Team

        Operation to delete a Team by Team ID.
         :param teamid: Specify the Team ID.
         :return: None"""
        self.validate_path_param(teamid)
        await self.delete(f"/api/V1/deleteteam/{teamid}")
        return None

    async def GetTeamsV1(
        self,
    ) -> cherwell_pydantic_api._generated.api.models.Trebuchet.WebApi.DataContracts.Teams.TeamsResponse:
        """Get all available Teams

        Operation to get IDs and names for all available Teams.
         :return: cherwell_pydantic_api._generated.api.models.Trebuchet.WebApi.DataContracts.Teams.TeamsResponse
        """
        response = await self.get("/api/V1/getteams")
        return parse_obj_as(
            cherwell_pydantic_api._generated.api.models.Trebuchet.WebApi.DataContracts.Teams.TeamsResponse,
            response,
        )

    async def GetTeamsV2(
        self,
    ) -> cherwell_pydantic_api._generated.api.models.Trebuchet.WebApi.DataContracts.Teams.TeamsV2Response:
        """Get all available Teams

        Operation to get IDs and names for all available Teams.
         :return: cherwell_pydantic_api._generated.api.models.Trebuchet.WebApi.DataContracts.Teams.TeamsV2Response
        """
        response = await self.get("/api/V2/getteams")
        return parse_obj_as(
            cherwell_pydantic_api._generated.api.models.Trebuchet.WebApi.DataContracts.Teams.TeamsV2Response,
            response,
        )

    async def GetTeamV1(
        self,
        teamid: str,
    ) -> cherwell_pydantic_api._generated.api.models.Trebuchet.WebApi.DataContracts.Teams.TeamResponse:
        """Get a team by its TeamId

        Operation to get Team Info for a  single Team using its Team ID. To get a Team's internal ID, use "Get all available Teams." Note that TeamType has two possible values, where TeamType = 0 for User (CSM Users), or TeamType = 1 for Workgroup (CSM Customers).
         :param teamid: The Team ID of the Team to get.
         :return: cherwell_pydantic_api._generated.api.models.Trebuchet.WebApi.DataContracts.Teams.TeamResponse
        """
        self.validate_path_param(teamid)
        response = await self.get(f"/api/V1/getteam/{teamid}")
        return parse_obj_as(
            cherwell_pydantic_api._generated.api.models.Trebuchet.WebApi.DataContracts.Teams.TeamResponse,
            response,
        )

    async def GetUsersTeamsV1(
        self,
        userRecordId: str,
    ) -> cherwell_pydantic_api._generated.api.models.Trebuchet.WebApi.DataContracts.Teams.TeamsResponse:
        """Get Team assignments for a user

        Operation to get Team assignments for a user. To get record IDs, use "Get a user by login ID" or "Get a user by public id."
         :param userRecordId: Specify the user record ID.
         :return: cherwell_pydantic_api._generated.api.models.Trebuchet.WebApi.DataContracts.Teams.TeamsResponse
        """
        self.validate_path_param(userRecordId)
        response = await self.get(f"/api/V1/getusersteams/userrecordid/{userRecordId}")
        return parse_obj_as(
            cherwell_pydantic_api._generated.api.models.Trebuchet.WebApi.DataContracts.Teams.TeamsResponse,
            response,
        )

    async def GetUsersTeamsV2(
        self,
        userRecordId: str,
    ) -> cherwell_pydantic_api._generated.api.models.Trebuchet.WebApi.DataContracts.Teams.TeamsV2Response:
        """Get Team assignments for a user

        Operation to get Team assignments for a user. To get record IDs, use "Get a user by login ID" or "Get a user by public id."
         :param userRecordId: Specify the user record ID.
         :return: cherwell_pydantic_api._generated.api.models.Trebuchet.WebApi.DataContracts.Teams.TeamsV2Response
        """
        self.validate_path_param(userRecordId)
        response = await self.get(f"/api/V2/getusersteams/userrecordid/{userRecordId}")
        return parse_obj_as(
            cherwell_pydantic_api._generated.api.models.Trebuchet.WebApi.DataContracts.Teams.TeamsV2Response,
            response,
        )

    async def GetWorkgroupsV1(
        self,
    ) -> cherwell_pydantic_api._generated.api.models.Trebuchet.WebApi.DataContracts.Teams.TeamsResponse:
        """Get all available Workgroups

        Operation to get IDs and names for all available Workgroups.
         :return: cherwell_pydantic_api._generated.api.models.Trebuchet.WebApi.DataContracts.Teams.TeamsResponse
        """
        response = await self.get("/api/V1/getworkgroups")
        return parse_obj_as(
            cherwell_pydantic_api._generated.api.models.Trebuchet.WebApi.DataContracts.Teams.TeamsResponse,
            response,
        )

    async def GetWorkgroupsV2(
        self,
    ) -> cherwell_pydantic_api._generated.api.models.Trebuchet.WebApi.DataContracts.Teams.TeamsV2Response:
        """Get all available Workgroups

        Operation to get IDs and names for all available Workgroups.
         :return: cherwell_pydantic_api._generated.api.models.Trebuchet.WebApi.DataContracts.Teams.TeamsV2Response
        """
        response = await self.get("/api/V2/getworkgroups")
        return parse_obj_as(
            cherwell_pydantic_api._generated.api.models.Trebuchet.WebApi.DataContracts.Teams.TeamsV2Response,
            response,
        )

    async def RemoveUserFromTeamV1(
        self,
        teamId: str,
        userrecordid: str,
    ) -> None:
        """Operation to remove a User from a Team.

        Operation to remove a User from a Team. To get the User's record ID, use "Get a User by login ID" or "Get a User by public ID." To get a Team's internal ID, use "Get all available Teams."
         :param teamId: Specify the internal ID of the Team.
         :param userrecordid: Specify the record ID of the User to remove.
         :return: None"""
        self.validate_path_param(teamId)
        self.validate_path_param(userrecordid)
        await self.delete(
            f"/api/V1/removeuserfromteam/teamid/{teamId}/userrecordid/{userrecordid}"
        )
        return None

    async def RemoveUserFromTeamV2(
        self,
        teamId: str,
        userrecordid: str,
    ) -> cherwell_pydantic_api._generated.api.models.Trebuchet.WebApi.DataContracts.Teams.RemoveUserFromTeamResponse:
        """Operation to remove a User from a Team.

        Operation to remove a User from a Team. To get the User's record ID, use "Get a User by login ID" or "Get a User by public ID." To get a Team's internal ID, use "Get all available Teams."
         :param teamId: Specify the internal ID of the Team.
         :param userrecordid: Specify the record ID of the User to remove.
         :return: cherwell_pydantic_api._generated.api.models.Trebuchet.WebApi.DataContracts.Teams.RemoveUserFromTeamResponse
        """
        self.validate_path_param(teamId)
        self.validate_path_param(userrecordid)
        response = await self.delete(
            f"/api/V2/removeuserfromteam/teamid/{teamId}/userrecordid/{userrecordid}"
        )
        return parse_obj_as(
            cherwell_pydantic_api._generated.api.models.Trebuchet.WebApi.DataContracts.Teams.RemoveUserFromTeamResponse,
            response,
        )

    async def RemoveCustomerFromWorkgroupV1(
        self,
        workgroupid: str,
        customerrecordid: str,
    ) -> cherwell_pydantic_api._generated.api.models.Trebuchet.WebApi.DataContracts.Teams.RemoveCustomerFromWorkgroupResponse:
        """Remove a customer from a Workgroup

        Operation to remove a Customer from a Workgroup.  To remove, specify the Workgroup ID and the Customer Record ID.
         :param workgroupid: Specify the Workgroup ID.
         :param customerrecordid: Specify the Customer record ID.
         :return: cherwell_pydantic_api._generated.api.models.Trebuchet.WebApi.DataContracts.Teams.RemoveCustomerFromWorkgroupResponse
        """
        self.validate_path_param(workgroupid)
        self.validate_path_param(customerrecordid)
        response = await self.delete(
            f"/api/V1/removecustomerfromworkgroup/workgroupid/{workgroupid}/customerrecordid/{customerrecordid}"
        )
        return parse_obj_as(
            cherwell_pydantic_api._generated.api.models.Trebuchet.WebApi.DataContracts.Teams.RemoveCustomerFromWorkgroupResponse,
            response,
        )

    async def SaveTeamV1(
        self,
        request: cherwell_pydantic_api._generated.api.models.Trebuchet.WebApi.DataContracts.Teams.TeamSaveRequest,
    ) -> cherwell_pydantic_api._generated.api.models.Trebuchet.WebApi.DataContracts.Teams.TeamSaveResponse:
        """Create or update a team

        Operation to create or update a Team or Workgroup.
         :param request: Request object to create Teams or Workgroups. To create a Team, use teamType and teamName. To update a team, use teamID. Team type values must be User or CustomerWorkgroup. The teamType cannot be changed for existing Teams or Workgroups.
         :return: cherwell_pydantic_api._generated.api.models.Trebuchet.WebApi.DataContracts.Teams.TeamSaveResponse
        """
        response = await self.post_body(
            "/api/V1/saveteam", content=request.json(exclude_unset=True, by_alias=True)
        )
        return parse_obj_as(
            cherwell_pydantic_api._generated.api.models.Trebuchet.WebApi.DataContracts.Teams.TeamSaveResponse,
            response,
        )

    async def SaveTeamMemberV1(
        self,
        request: cherwell_pydantic_api._generated.api.models.Trebuchet.WebApi.DataContracts.Teams.SaveTeamMemberRequest,
    ) -> cherwell_pydantic_api._generated.api.models.Trebuchet.WebApi.DataContracts.Teams.SaveTeamMemberResponse:
        """Add or Update a team member

        Operation to add or update a Team Member. To add or update, specify User ID, Team ID, and if Team Manager.   Optionally, set the Team as the User's default Team.
         :param request: The request object to add or update a Team Member. User recID specifies the User to add or update. TeamId specifies the Team to update. IsTeamManager specifies whether the User is a Team Manager, and SetAsDefaultTeam specifies whether to set this Team as the User's default team. UserRecId, TeamId, and IsTeamManager are required.
         :return: cherwell_pydantic_api._generated.api.models.Trebuchet.WebApi.DataContracts.Teams.SaveTeamMemberResponse
        """
        response = await self.post_body(
            "/api/V1/saveteammember",
            content=request.json(exclude_unset=True, by_alias=True),
        )
        return parse_obj_as(
            cherwell_pydantic_api._generated.api.models.Trebuchet.WebApi.DataContracts.Teams.SaveTeamMemberResponse,
            response,
        )

    async def SaveWorkgroupMemberV1(
        self,
        request: cherwell_pydantic_api._generated.api.models.Trebuchet.WebApi.DataContracts.Teams.SaveWorkgroupMemberRequest,
    ) -> cherwell_pydantic_api._generated.api.models.Trebuchet.WebApi.DataContracts.Teams.SaveWorkgroupMemberResponse:
        """Save the membership status of a Workgroup member.

        Operation to add or update a Workgroup Member.  To add or update, specify Customer Record ID, Workgroup ID, and if Workgroup Manager.
         :param request: The request object to add or update a Workgroup Member. CustomerRecordId specifies the Customer to add or update. WorkgroupId specifies the Workgroup to update. CustomerIsWorkgroupManager specifies whether the Customer is a Workgroup Manager.
         :return: cherwell_pydantic_api._generated.api.models.Trebuchet.WebApi.DataContracts.Teams.SaveWorkgroupMemberResponse
        """
        response = await self.post_body(
            "/api/V1/saveworkgroupmember",
            content=request.json(exclude_unset=True, by_alias=True),
        )
        return parse_obj_as(
            cherwell_pydantic_api._generated.api.models.Trebuchet.WebApi.DataContracts.Teams.SaveWorkgroupMemberResponse,
            response,
        )
